@@ -51,6 +51,7 @@
  * @date 2014-04-01  Compatible with v0.7.0 new interface.
  * @date 2015-05-14  Compatible with v0.8.0 new interface.
  * @date 2016-09-10  Compatible with the interface at version 0.10.0
+ * @date 2017-09-20  Compatible with the interface at version 1.0
  */
 #include  <stdio.h>
 #include  <stdlib.h>
@@ -105,9 +106,12 @@ int main(int argc, char **argv)
   int           info;
   int           outputFileIndex;
 
-
-
+  #ifdef WITH_SYMPACK
+  symPACK_Init(&argc, &argv);
+  #else
   MPI_Init( &argc, &argv );
+  #endif
+
   MPI_Comm_rank( MPI_COMM_WORLD, &mpirank );
   MPI_Comm_size( MPI_COMM_WORLD, &mpisize );
 
@@ -118,11 +122,12 @@ int main(int argc, char **argv)
   nprow               = 1;
   npcol               = 1;
   Hfile               = "lap2dr.matrix";
+//  Hfile               = "H.csc";
   Sfile               = "";
   isFormatted         = 1;
   isSIdentity         = 1;
 #else
-#if 1
+#if 0
   numElectronExact    = 7000.0;
   nprow               = 8;
   npcol               = 8;
@@ -268,11 +273,16 @@ int main(int argc, char **argv)
   options.maxPEXSIIter   = 1;
   options.verbosity = 1;
   options.deltaE   = 20.0;
-  options.numPole  = 40;
-  options.temperature  = 0.0019; // 300K
+  options.numPole  = 80;
+  options.temperature  = 0.00095; // 300K
   options.muPEXSISafeGuard  = 0.2; 
-  options.numElectronPEXSITolerance = 0.001;
+  options.numElectronPEXSITolerance = 0.0001;
   options.isSymbolicFactorize = 1;
+  #ifdef WITH_SYMPACK
+  options.solver = 1;
+  #endif
+  options.symmetricStorage = 1;
+
 
   /* Set the outputFileIndex to be the pole index */
   /* The first processor for each pole outputs information */
@@ -441,7 +451,12 @@ int main(int argc, char **argv)
 
   
   MPI_Comm_free( &readComm );
+  #ifdef WITH_SYMPACK
+  symPACK_Finalize();
+  #else
   MPI_Finalize();
+  #endif
+
 
   return 0;
 }
