@@ -203,12 +203,6 @@ int main(int argc, char **argv)
       plan,
       &info );
   
-  if( mpirank == 0 ){ 
-    printf("\nAll calculation is finished. Exit the program.\n");
-  }
-
-
-
   /* Deallocate memory */
   free( colptrLocal );
   free( rowindLocal );
