@@ -4,12 +4,12 @@
 /// @date 2014-07-04
 
 #include <mpi.h>
+#include <omp.h>
 
 #include <complex>
 #include <string>
 #include <sstream>
 
-// Libraries from NGCHOL BEGIN
 #include  "ppexsi.hpp"
 #include "pexsi/timer.h"
 
@@ -17,7 +17,7 @@
 #include "pexsi/sympack_interf.hpp"
 #include <memory>
 
-//#define _MYCOMPLEX_
+#define _MYCOMPLEX_
 
 typedef double ISCALAR;
 #ifdef _MYCOMPLEX_
@@ -33,7 +33,7 @@ using namespace PEXSI;
 using namespace std;
 
 void Usage(){
-  std::cout << "Usage" << std::endl << "run_pselinv_sympack -H <symmetric Hfile> -S [symmetric Sfile] -inf [H/S file format: CSC,HARWELL_BOEING] -r [nprow] -c [npcol] -rshift [real shift] -ishift [imaginary shift] -ToDist [doToDist] -Diag [doDiag]" << std::endl;
+  std::cout << "Usage" << std::endl << "run_pselinv_sympack -H <symmetric Hfile> -S [symmetric Sfile] -inf [H/S file format: CSC,HARWELL_BOEING] -r [nprow] -c [npcol] -rshift [real shift] -ishift [imaginary shift] -ToDist [doToDist] -Diag [doDiag] -SS [symmetricStorage]" << std::endl;
 }
 
 
@@ -42,8 +42,6 @@ int main(int argc, char **argv)
   int mpisize;
   int mpirank;
 
-  MPI_Init(&argc,&argv);
-
   symPACK_Init(&argc,&argv);
 
   //Create a communicator with npcol*nprow processors
@@ -67,7 +65,7 @@ int main(int argc, char **argv)
   statusOFS.open( ss.str().c_str() );
 
 #if defined (PROFILE) || defined(PMPI) || defined(USE_TAU)
-      TAU_PROFILE_SET_CONTEXT(world_comm);
+  TAU_PROFILE_SET_CONTEXT(world_comm);
 #endif
 
   if(argc<3){
@@ -75,7 +73,7 @@ int main(int argc, char **argv)
       Usage();
     }
   }
-    
+
 
   try{
 
@@ -154,6 +152,59 @@ int main(int argc, char **argv)
         << std::endl << std::endl;
     }
 
+    Int symmetricStorage = 0;
+    if( options.find("-SS") != options.end() ){ 
+      symmetricStorage = atoi(options["-SS"].c_str());
+    }
+    else{
+      statusOFS << "-SS option is not given. " 
+        << "Do not use symmetric storage." 
+        << std::endl << std::endl;
+    }
+
+
+
+    int doFacto = 1;
+    if( options.find("-F") != options.end() ){ 
+      doFacto= atoi(options["-F"].c_str());
+    }
+
+    int doSymbolic = 1;
+    if( options.find("-Symb") != options.end() ){ 
+      doSymbolic= atoi(options["-Symb"].c_str());
+    }
+
+    int doConvert = 1;
+    if( options.find("-C") != options.end() ){ 
+      doConvert= atoi(options["-C"].c_str());
+    }
+
+    int doDistribute = 1;
+    if( options.find("-D") != options.end() ){ 
+      doDistribute= atoi(options["-D"].c_str());
+    }
+
+    Int doSelinv = 1;
+    if( options.find("-Sinv") != options.end() ){ 
+      doSelinv = atoi(options["-Sinv"].c_str());
+    }
+
+    if(doSelinv){
+      doConvert=1;
+    }
+
+    if(doConvert){
+      doFacto=1;
+    }
+
+    if(doFacto){
+      doDistribute=1;
+    }
+
+    if(doDistribute){
+      doSymbolic = 1;
+    }
+
     Real rshift = 0.0, ishift = 0.0;
     if( options.find("-rshift") != options.end() ){ 
       rshift = atof(options["-rshift"].c_str());
@@ -175,6 +226,18 @@ int main(int argc, char **argv)
     }
 
 
+
+    std::string refinement;
+    if( options.find("-refine") != options.end() ){ 
+      refinement = options["-refine"];
+    }
+    else{
+      refinement = "NO";
+    }  
+
+
+
+
 #ifdef _MYCOMPLEX_
     int isComplex = 0;
     if( options.find("-Complex") != options.end() ){ 
@@ -197,14 +260,22 @@ int main(int argc, char **argv)
 
     symPACK::symPACKOptions optionsFact;
     optionsFact.NpOrdering = numProcSymbFact;
-    optionsFact.decomposition = symPACK::LDL;
+    optionsFact.decomposition = symPACK::DecompositionType::LDL;
     optionsFact.orderingStr = ColPerm;
     optionsFact.MPIcomm = workcomm;
     optionsFact.verbose=0;
+    optionsFact.order_refinement_str = refinement;
 
-//    optionsFact.load_balance_str = "NNZ";
-//    optionsFact.maxIsend = 100;
-//    optionsFact.relax.SetMaxSize(300);
+    //  optionsFact.MPIcomm = worldcomm;
+    optionsFact.factorization = symPACK::FANBOTH;
+    optionsFact.factorization = symPACK::FANOUT;
+    //symPACK::Multithreading::NumThread = omp_get_max_threads();
+    //optionsFact.print_stats=false;
+
+    //    optionsFact.load_balance_str = "NNZ";
+    //    optionsFact.maxIsend = 100;
+    //    optionsFact.relax.SetMaxSize(0);
+    //    optionsFact.relax.SetMaxSize(60);
 
 
     //Initialize UPCXX for symPACK
@@ -227,13 +298,13 @@ int main(int argc, char **argv)
       //symPACK::symPACKMatrix<SCALAR>*  symPACKMat = NULL;
 #ifdef _MYCOMPLEX_
       if(isComplex){
-        symPACK::DistSparseMatrix<Complex> HMat(workcomm);
-        symPACK::ReadMatrix<Complex,Complex>(Hfile, format, HMat);
+        symPACK::DistSparseMatrix<Complex> HMatComplex(workcomm);
+        symPACK::ReadMatrix<Complex,Complex>(Hfile, format, HMatComplex);
 
         //Build AMat
         symPACK::DistSparseMatrix<Complex> SMat(workcomm);
 
-        const symPACK::DistSparseMatrixGraph & Hgraph = HMat.GetLocalGraph();
+        const symPACK::DistSparseMatrixGraph & Hgraph = HMatComplex.GetLocalGraph();
         // Get the diagonal indices for H and save it n diagIdxLocal_
         std::vector<Int>  diagIdxLocal;
         { 
@@ -242,7 +313,7 @@ int main(int argc, char **argv)
           Int firstCol         = Hgraph.LocalFirstVertex();
 
           diagIdxLocal.clear();
-          diagIdxLocal.reserve( HMat.size );
+          diagIdxLocal.reserve( HMatComplex.size );
           for( symPACK::Idx j = 0; j < numColLocal; j++ ){
             symPACK::Idx jcol = firstCol + j + 1;
             for( symPACK::Ptr i = Hgraph.colptr[j]-1; 
@@ -257,26 +328,26 @@ int main(int argc, char **argv)
 
         GetTime( timeSta );
 
-        AMat.size          = HMat.size;
-        AMat.nnz           = HMat.nnz;
-        AMat.SetLocalGraph(HMat.GetLocalGraph());
-        AMat.nzvalLocal.resize( HMat.nzvalLocal.size() );
+        AMat.size          = HMatComplex.size;
+        AMat.nnz           = HMatComplex.nnz;
+        AMat.SetLocalGraph(HMatComplex.GetLocalGraph());
+        AMat.nzvalLocal.resize( HMatComplex.nzvalLocal.size() );
 
         SCALAR *ptr0 = AMat.nzvalLocal.data();
-        Complex *ptr1 = HMat.nzvalLocal.data();
+        Complex *ptr1 = HMatComplex.nzvalLocal.data();
         Complex *ptr2 = SMat.nzvalLocal.data();
 
 
         if( SMat.size != 0 ){
           // S is not an identity matrix
-          for( Int i = 0; i < HMat.nzvalLocal.size(); i++ ){
-            AMat.nzvalLocal[i] = HMat.nzvalLocal[i] - zshift * SMat.nzvalLocal[i];
+          for( Int i = 0; i < HMatComplex.nzvalLocal.size(); i++ ){
+            AMat.nzvalLocal[i] = HMatComplex.nzvalLocal[i] - zshift * SMat.nzvalLocal[i];
           }
         }
         else{
           // S is an identity matrix
           for( Int i = 0; i < Hgraph.LocalEdgeCount(); i++ ){
-            AMat.nzvalLocal[i] = HMat.nzvalLocal[i];
+            AMat.nzvalLocal[i] = HMatComplex.nzvalLocal[i];
           }
 
           for( Int i = 0; i < diagIdxLocal.size(); i++ ){
@@ -298,8 +369,8 @@ int main(int argc, char **argv)
       else
 #endif
       {
-        symPACK::DistSparseMatrix<ISCALAR> HMat(workcomm);
-        symPACK::ReadMatrix<ISCALAR,ISCALAR>(Hfile, format, HMat);
+        symPACK::DistSparseMatrix<ISCALAR> HMatReal(workcomm);
+        symPACK::ReadMatrix<ISCALAR,ISCALAR>(Hfile, format, HMatReal);
 
         //Build AMat
         symPACK::DistSparseMatrix<ISCALAR> SMat(workcomm);
@@ -317,7 +388,7 @@ int main(int argc, char **argv)
 
 
 
-        const symPACK::DistSparseMatrixGraph & Hgraph = HMat.GetLocalGraph();
+        const symPACK::DistSparseMatrixGraph & Hgraph = HMatReal.GetLocalGraph();
         // Get the diagonal indices for H and save it n diagIdxLocal_
         std::vector<Int>  diagIdxLocal;
         { 
@@ -326,7 +397,7 @@ int main(int argc, char **argv)
           Int firstCol         = Hgraph.LocalFirstVertex();
 
           diagIdxLocal.clear();
-          diagIdxLocal.reserve( HMat.size );
+          diagIdxLocal.reserve( HMatReal.size );
           for( symPACK::Idx j = 0; j < numColLocal; j++ ){
             symPACK::Idx jcol = firstCol + j + 1;
             for( symPACK::Ptr i = Hgraph.colptr[j]-1; 
@@ -341,26 +412,26 @@ int main(int argc, char **argv)
 
         GetTime( timeSta );
 
-        AMat.size          = HMat.size;
-        AMat.nnz           = HMat.nnz;
-        AMat.SetLocalGraph(HMat.GetLocalGraph());
-        AMat.nzvalLocal.resize( HMat.nzvalLocal.size() );
+        AMat.size          = HMatReal.size;
+        AMat.nnz           = HMatReal.nnz;
+        AMat.SetLocalGraph(HMatReal.GetLocalGraph());
+        AMat.nzvalLocal.resize( HMatReal.nzvalLocal.size() );
 
         SCALAR *ptr0 = AMat.nzvalLocal.data();
-        Real *ptr1 = HMat.nzvalLocal.data();
+        Real *ptr1 = HMatReal.nzvalLocal.data();
         Real *ptr2 = SMat.nzvalLocal.data();
 
 
         if( SMat.size != 0 ){
           // S is not an identity matrix
-          for( Int i = 0; i < HMat.nzvalLocal.size(); i++ ){
-            AMat.nzvalLocal[i] = HMat.nzvalLocal[i] - zshift * SMat.nzvalLocal[i];
+          for( Int i = 0; i < HMatReal.nzvalLocal.size(); i++ ){
+            AMat.nzvalLocal[i] = HMatReal.nzvalLocal[i] - zshift * SMat.nzvalLocal[i];
           }
         }
         else{
           // S is an identity matrix
           for( Int i = 0; i < Hgraph.LocalEdgeCount(); i++ ){
-            AMat.nzvalLocal[i] = HMat.nzvalLocal[i];
+            AMat.nzvalLocal[i] = HMatReal.nzvalLocal[i];
           }
 
           for( Int i = 0; i < diagIdxLocal.size(); i++ ){
@@ -379,165 +450,181 @@ int main(int argc, char **argv)
       }
 
       PMatrix<SCALAR> * pMat = NULL; 
+      FactorizationOptions factOpt;
+      PSelInvOptions selInvOpt;
       GridType *gPtr = new GridType( workcomm, nprow, npcol );
       SuperNodeType *superPtr = new SuperNodeType();
       {
-        GetTime( timeTotalFactorizationSta );
-        GetTime( timeSta );
-        std::unique_ptr<symPACK::symPACKMatrix<SCALAR> > symPACKMat(new symPACK::symPACKMatrix<SCALAR>());
+        std::shared_ptr<symPACK::symPACKMatrix<SCALAR> > symPACKMat(new symPACK::symPACKMatrix<SCALAR>());
+        if(doSymbolic){
+          GetTime( timeSta );
+          symPACKMat->Init(optionsFact);
+          symPACKMat->SymbolicFactorization(AMat);
+          GetTime( timeEnd );
+          if( mpirank == 0 )
+            cout << "Time for symbolic factorization is " << timeEnd - timeSta << " sec" << endl; 
+        }
 
-        symPACKMat->Init(optionsFact);
-        symPACKMat->SymbolicFactorization(AMat);
-        GetTime( timeEnd );
-        if( mpirank == 0 )
-          cout << "Time for symbolic factorization is " << timeEnd - timeSta << " sec" << endl; 
-        GetTime( timeSta );
-        symPACKMat->DistributeMatrix(AMat);
-        GetTime( timeEnd );
-        if( mpirank == 0 )
-          cout << "Time for distribution is " << timeEnd - timeSta << " sec" << endl; 
-        GetTime( timeSta );
-        symPACKMat->Factorize();
-        GetTime( timeEnd );
+        if(doDistribute){
+          GetTime( timeTotalFactorizationSta );
+          GetTime( timeSta );
+          symPACKMat->DistributeMatrix(AMat);
+          GetTime( timeEnd );
+          if( mpirank == 0 )
+            cout << "Time for distribution is " << timeEnd - timeSta << " sec" << endl; 
+        }
 
-        if( mpirank == 0 )
-          cout << "Time for factorization is " << timeEnd - timeSta << " sec" << endl; 
+        //MPI_Barrier(workcomm);
 
-        GetTime( timeTotalFactorizationEnd );
-        if( mpirank == 0 )
-          cout << "Time for total factorization is " << timeTotalFactorizationEnd - timeTotalFactorizationSta<< " sec" << endl; 
+        if(doFacto){
+          GetTime( timeSta );
+          symPACKMat->Factorize();
+          GetTime( timeEnd );
 
-        //      symPACKMat->DumpMatlab();
+          if( mpirank == 0 )
+            cout << "Time for factorization is " << timeEnd - timeSta << " sec" << endl; 
 
+          GetTime( timeTotalFactorizationEnd );
+          if( mpirank == 0 )
+            cout << "Time for total factorization is " << timeTotalFactorizationEnd - timeTotalFactorizationSta<< " sec" << endl; 
+          //      symPACKMat->DumpMatlab();
+
+        }
 
         GetTime( timeTotalSelInvSta );
 
-        FactorizationOptions factOpt;
         factOpt.ColPerm = ColPerm;
         factOpt.Symmetric = 1;
 
-        PSelInvOptions selInvOpt;
-        selInvOpt.maxPipelineDepth = -1;
+        selInvOpt.maxPipelineDepth = maxPipelineDepth;
+        selInvOpt.symmetricStorage = symmetricStorage;
 
-        GetTime( timeSta );
-        symPACKMatrixToSuperNode( *symPACKMat, *superPtr );
+        if (doConvert){
+          GetTime( timeSta );
+          symPACKMatrixToSuperNode( *symPACKMat, *superPtr );
 
 
-        pMat = PMatrix<SCALAR>::Create(gPtr,superPtr, &selInvOpt, &factOpt);
-        symPACKMatrixToPMatrix( *symPACKMat, *pMat );
-        GetTime( timeEnd );
+          pMat = PMatrix<SCALAR>::Create(gPtr,superPtr, &selInvOpt, &factOpt);
+          symPACKMatrixToPMatrix( *symPACKMat, *pMat );
+          GetTime( timeEnd );
 
-        if( mpirank == 0 )
-          cout << "Time for converting symPACK matrix to PMatrix is " << timeEnd  - timeSta << endl;
+          if( mpirank == 0 )
+            cout << "Time for converting symPACK matrix to PMatrix is " << timeEnd  - timeSta << endl;
+        }
 
+        //MPI_Barrier(workcomm);
       }
- 
-      // Preparation for the selected inversion
-      GetTime( timeSta );
-      pMat->ConstructCommunicationPattern();
-      GetTime( timeEnd );
-      if( mpirank == 0 )
-        cout << "Time for constructing the communication pattern is " << timeEnd  - timeSta << endl;
 
-      GetTime( timeSta );
-      pMat->PreSelInv();
-      GetTime( timeEnd );
-      if( mpirank == 0 )
-        cout << "Time for pre-selected inversion is " << timeEnd  - timeSta << endl;
-      statusOFS << "Time for pre-selected inversion is " << timeEnd  - timeSta << endl;
-
-      GetTime( timeSta );
-      pMat->SelInv();
-      GetTime( timeEnd );
-      GetTime( timeTotalSelInvEnd );
-      if( mpirank == 0 )
-        cout << "Time for numerical selected inversion is " << timeEnd  - timeSta << endl;
-      statusOFS << "Time for numerical selected inversion is " << timeEnd  - timeSta << endl;
-
-
-      if( mpirank == 0 )
-        cout << "Time for total selected inversion is " << timeTotalSelInvEnd  - timeTotalSelInvSta << endl;
-
-
-      if( doDiag ){
-        NumVec<SCALAR> diag;
+      if (doSelinv>0){
+        // Preparation for the selected inversion
+        GetTime( timeSta );
+        pMat->ConstructCommunicationPattern();
+        GetTime( timeEnd );
+        if( mpirank == 0 )
+          cout << "Time for constructing the communication pattern is " << timeEnd  - timeSta << endl;
 
         GetTime( timeSta );
-        pMat->GetDiagonal( diag );
+        pMat->PreSelInv();
         GetTime( timeEnd );
+        if( mpirank == 0 )
+          cout << "Time for pre-selected inversion is " << timeEnd  - timeSta << endl;
+        statusOFS << "Time for pre-selected inversion is " << timeEnd  - timeSta << endl;
+
+        GetTime( timeSta );
+        pMat->SelInv();
+        GetTime( timeEnd );
+        GetTime( timeTotalSelInvEnd );
+        if( mpirank == 0 )
+          cout << "Time for numerical selected inversion is " << timeEnd  - timeSta << endl;
+        statusOFS << "Time for numerical selected inversion is " << timeEnd  - timeSta << endl;
+
 
         if( mpirank == 0 )
-          cout << "Time for getting the diagonal is " << timeEnd  - timeSta << endl;
+          cout << "Time for total selected inversion is " << timeTotalSelInvEnd  - timeTotalSelInvSta << endl;
+
+#ifdef _PRINT_STATS_
+        double flops = pMat->GetTotalFlops();
+        if( mpirank == 0 )
+          cout << "Total FLOPs for selected inversion is " << flops << endl;
+#endif
+
+        if( doDiag ){
+          NumVec<SCALAR> diag;
+
+          GetTime( timeSta );
+          pMat->GetDiagonal( diag );
+          GetTime( timeEnd );
+
+          if( mpirank == 0 )
+            cout << "Time for getting the diagonal is " << timeEnd  - timeSta << endl;
 
 
-        if( mpirank == 0 ){
-          statusOFS << std::endl << "Diagonal (pipeline) of inverse in natural order: " << std::endl << diag << std::endl;
-          ofstream ofs("diag");
-          if( !ofs.good() ) 
-            ErrorHandling("file cannot be opened.");
-          serialize( diag, ofs, NO_MASK );
-          ofs.close();
+          if( mpirank == 0 ){
+            statusOFS << std::endl << "Diagonal (pipeline) of inverse in natural order: " << std::endl << diag << std::endl;
+            ofstream ofs("diag");
+            if( !ofs.good() ) 
+              ErrorHandling("file cannot be opened.");
+            serialize( diag, ofs, NO_MASK );
+            ofs.close();
+          }
+        }
+
+        if(doToDist){
+          //Expand AMat to unsymmetric storage
+          GetTime( timeSta );
+          AMat.ExpandSymmetric();
+          AMat.SortGraph();
+          AMat.GetLocalGraph().SetBaseval(1);
+          GetTime( timeEnd );
+
+          if( mpirank == 0 )
+            cout << "Matrix A has been expanded in " << timeEnd  - timeSta << endl;
+
+          DistSparseMatrix<SCALAR> AMatPattern;
+          AMatPattern.size = AMat.size;
+          AMatPattern.nnz = AMat.nnz;
+          AMatPattern.colptrLocal.Resize(AMat.GetLocalGraph().colptr.size());
+          std::copy(AMat.GetLocalGraph().colptr.begin(), AMat.GetLocalGraph().colptr.end(), &AMatPattern.colptrLocal[0]);
+          AMatPattern.rowindLocal.Resize(AMat.GetLocalGraph().rowind.size());
+          std::copy(AMat.GetLocalGraph().rowind.begin(), AMat.GetLocalGraph().rowind.end(), &AMatPattern.rowindLocal[0]);
+          AMatPattern.nnzLocal = AMat.GetLocalGraph().rowind.size();
+
+          // Convert to DistSparseMatrix in the 2nd format and get the diagonal
+          DistSparseMatrix<SCALAR> Ainv;
+
+          GetTime( timeSta );
+          pMat->PMatrixToDistSparseMatrix( AMatPattern, Ainv );
+          GetTime( timeEnd );
+
+
+          if( mpirank == 0 )
+            cout << "Time for converting PMatrix to DistSparseMatrix (2nd format) is " << timeEnd  - timeSta << endl;
+
+          SCALAR traceLocal = ZERO<SCALAR>();
+          traceLocal = blas::Dotu( Ainv.nnzLocal, Ainv.nzvalLocal.Data(), 1,
+              AMat.nzvalLocal.data(), 1 );
+
+
+          SCALAR trace = ZERO<SCALAR>();
+          mpi::Allreduce( &traceLocal, &trace, 1, MPI_SUM, workcomm );
+
+          if( mpirank == 0 ){
+            cout << "A.size = "  << AMat.size << endl;
+            cout << std::endl << "Tr[Ainv * AMat] = " <<  trace << std::endl;
+            statusOFS << std::endl << "Tr[Ainv * AMat] = " << std::endl << trace << std::endl;
+
+            cout << std::endl << "|N - Tr[Ainv * AMat]| = " << std::abs( SCALAR(AMat.size) - trace ) << std::endl;
+            statusOFS << std::endl << "|N - Tr[Ainv * AMat]| = " << std::abs( SCALAR(AMat.size) - trace ) << std::endl;
+          }
         }
       }
 
-      if(doToDist){
-        //Expand AMat to unsymmetric storage
-        GetTime( timeSta );
-        AMat.ExpandSymmetric();
-        AMat.SortGraph();
-        AMat.GetLocalGraph().SetBaseval(1);
-        GetTime( timeEnd );
-
-        if( mpirank == 0 )
-          cout << "Matrix A has been expanded in " << timeEnd  - timeSta << endl;
-
-        DistSparseMatrix<SCALAR> AMatPattern;
-        AMatPattern.size = AMat.size;
-        AMatPattern.nnz = AMat.nnz;
-        AMatPattern.colptrLocal.Resize(AMat.GetLocalGraph().colptr.size());
-        std::copy(AMat.GetLocalGraph().colptr.begin(), AMat.GetLocalGraph().colptr.end(), &AMatPattern.colptrLocal[0]);
-        AMatPattern.rowindLocal.Resize(AMat.GetLocalGraph().rowind.size());
-        std::copy(AMat.GetLocalGraph().rowind.begin(), AMat.GetLocalGraph().rowind.end(), &AMatPattern.rowindLocal[0]);
-        AMatPattern.nnzLocal = AMat.GetLocalGraph().rowind.size();
-
-        // Convert to DistSparseMatrix in the 2nd format and get the diagonal
-        DistSparseMatrix<SCALAR> Ainv;
-
-        GetTime( timeSta );
-        pMat->PMatrixToDistSparseMatrix( AMatPattern, Ainv );
-        GetTime( timeEnd );
-
-
-        if( mpirank == 0 )
-          cout << "Time for converting PMatrix to DistSparseMatrix (2nd format) is " << timeEnd  - timeSta << endl;
-
-        SCALAR traceLocal = ZERO<SCALAR>();
-        traceLocal = blas::Dotu( Ainv.nnzLocal, Ainv.nzvalLocal.Data(), 1,
-            AMat.nzvalLocal.data(), 1 );
-
-
-        SCALAR trace = ZERO<SCALAR>();
-        mpi::Allreduce( &traceLocal, &trace, 1, MPI_SUM, workcomm );
-
-        if( mpirank == 0 ){
-          cout << "A.size = "  << AMat.size << endl;
-          cout << std::endl << "Tr[Ainv * AMat] = " <<  trace << std::endl;
-          statusOFS << std::endl << "Tr[Ainv * AMat] = " << std::endl << trace << std::endl;
-
-          cout << std::endl << "|N - Tr[Ainv * AMat]| = " << std::abs( SCALAR(AMat.size) - trace ) << std::endl;
-          statusOFS << std::endl << "|N - Tr[Ainv * AMat]| = " << std::abs( SCALAR(AMat.size) - trace ) << std::endl;
-        }
-      }
-
-
       delete pMat;
 
-
       delete superPtr;
       delete gPtr;
     }
-//    delete symPACK::logfileptr;
-    statusOFS.close();
+    //    delete symPACK::logfileptr;
 
   }
   catch( std::exception& e )
@@ -546,10 +633,10 @@ int main(int argc, char **argv)
       << e.what() << std::endl;
   }
 
-
-
+  statusOFS<<"Calling symPACK finalize"<<std::endl;
   symPACK_Finalize();
-//MPI_Finalize();
+  statusOFS.close();
+  //MPI_Finalize();
 
   return 0;
 }
