@@ -52,8 +52,9 @@ include 'mpif.h'
 
 integer(c_int) :: nrows, nnz, nnzLocal, numColLocal
 integer(c_int), allocatable, dimension(:) ::  colptrLocal, rowindLocal
-real(c_double), allocatable, dimension(:) ::  &
-  SnzvalLocal, RnzvalLocal, InzvalLocal, AnzvalLocal, AinvnzvalLocal
+real(c_double), allocatable, dimension(:) ::  RnzvalLocal, InzvalLocal
+complex(c_double), allocatable, dimension(:)::AinvnzvalLocal, AnzvalLocal,&
+        SnzvalLocal
 integer(c_int):: nprow, npcol, npSymbFact, outputFileIndex
 integer :: mpirank, mpisize, ierr
 double precision:: timeSta, timeEnd
@@ -98,8 +99,8 @@ allocate( colptrLocal( numColLocal + 1 ) )
 allocate( rowindLocal( nnzLocal ) )
 allocate( RnzvalLocal( nnzLocal ) )
 allocate( InzvalLocal( nnzLocal ) )
-allocate( AnzvalLocal( 2*nnzLocal ) )
-allocate( AinvnzvalLocal( 2*nnzLocal ) )
+allocate( AnzvalLocal( nnzLocal ) )
+allocate( AinvnzvalLocal( nnzLocal ) )
 
 ! Read the real part of the matrix
 call f_read_distsparsematrix_formatted (&
@@ -127,8 +128,7 @@ call f_read_distsparsematrix_formatted (&
 
 ! Form the matrix
 do i = 1, nnzLocal
-  AnzvalLocal(2*i-1) = RnzvalLocal(i)
-  AnzvalLocal(2*i)   = InzvalLocal(i)
+   AnzvalLocal(i)= dcmplx(RnzvalLocal(i), InzvalLocal(i))
 enddo
 
 ! Each processor outputs information
@@ -218,8 +218,8 @@ if( mpirank == 0 ) then
       irow = rowindLocal(i)
       if( irow == jcol ) then
         write(*,"(A,2I5,A,F15.10,A,F15.10,A)") &
-          "Ainv[", irow, irow, "]= ", AinvnzvalLocal(2*i-1), &
-          " + ", AinvnzvalLocal(2*i), " i"
+          "Ainv[", irow, irow, "]= ", dreal(AinvnzvalLocal(i)), &
+          " + ", dimag(AinvnzvalLocal(i)), " i"
       endif
     enddo
   enddo
