@@ -178,6 +178,10 @@ typedef intptr_t  PPEXSIPlan;
  * @brief Structure for the input parameters in DFT calculations.
  */
 typedef struct {
+    /** 
+     * @brief  NumSpin, default is 2.0
+     */ 
+    double        spin;  
     /** 
      * @brief  Temperature, in the same unit as H 
      */ 
@@ -258,15 +262,30 @@ typedef struct {
      */ 
     int           solver;
 
+    /** 
+     * @brief  Storage space used by the Selected Inversion algorithm for symmetric matrices.  
+     * - = 0   : Non symmetric storage.
+     * - = 1   : Symmetric storage (lower memory usage).
+     */ 
+    int           symmetricStorage;
+
+
     /** 
      * @brief  Ordering strategy for factorization and selected
-     * inversion.  
+     * inversion. When SuperLU is used:  
      * - = 0   : Parallel ordering using ParMETIS/PT-SCOTCH (PARMETIS
      *   option in SuperLU_DIST).
      * - = 1   : Sequential ordering using METIS (METIS_AT_PLUS_A
      *   option in SuperLU_DIST).
      * - = 2   : Multiple minimum degree ordering (MMD_AT_PLUS_A
      *   option in SuperLU_DIST).
+     * When symPACK is used:
+     * - = 0   : Parallel ordering using PT-SCOTCH.
+     * - = 1   : Sequential ordering using SCOTCH.
+     * - = 2   : Multiple minimum degree ordering.
+     * - = 3   : Approximate minimum degree ordering.
+     * - = 4   : Parallel ordering using PARMETIS.
+     * - = 5   : Sequential ordering using METIS.
      */ 
     int           ordering;
     /** 
@@ -295,6 +314,22 @@ typedef struct {
      * - = 1   : Factor transposed matrix.
      */ 
     int           transpose;
+    /** 
+     * @brief  The pole expansion method to be used.
+     * - = 1   : Cauchy Contour Integral method used.
+     * - = 2   : Moussa optimized method.
+     */ 
+    int           method;
+    /** 
+     * @brief  The point parallelizaion of PEXSI.
+     * - = 2  : Recommend two points parallelization
+     */ 
+    int           nPoints;
+    /** 
+     * @brief  The driver version of the PEXSI.
+     * - = 1.0.0 : Latest version is 1.0.0 
+     */ 
+    //char         driverVersion[10] ;//= "1.0.0";
     /** 
      * @brief  The level of output information.
      * - = 0   : No output.
@@ -692,6 +727,73 @@ void PPEXSICalculateFermiOperatorReal(
     double*           numElectronDrvMuPEXSI,
     int*              info );
 
+/**
+ * @brief Compute the density matrices and number of electrons for a
+ * given chemical potential.
+ * 
+ * This can be used as an "expert" interface for solving KSDFT with
+ * user-implemented heuristics strategies.
+ *
+ * **Input/Output**
+ *
+ * The input parameter options are controlled through the structure
+ * PPEXSIOptions.  The default value can be obtained through
+ * PPEXSISetDefaultOptions.
+
+ * The input H and S matrices should be given by loading functions
+ * (currently it is PPEXSILoadRealSymmetricHSMatrix).  The output
+ * matrices should be obtained from retrieving functions (currently it
+ * is PPEXSIRetrieveRealSymmetricDFTMatrix).
+ *
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[in] numElectronExact (global) Exact number of electrons, i.e.
+ * \f$N_e(\mu_{\mathrm{exact}})\f$.
+ * @param[in] options (global) Other input parameters for the DFT driver.  
+ * @param[out]  DMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero value
+ * of density matrix in CSC format.
+ * @param[out] EDMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero
+ * value of energy density matrix in CSC format.
+ * @param[out] FDMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero
+ * value of free energy density matrix in CSC format.
+ * @param[out] muPEXSI      (global) Chemical potential after the last
+ * iteration.
+ * - In the case that convergence is reached within maxPEXSIIter steps, the
+ * value of muPEXSI is the last mu used to achieve accuracy within
+ * numElectronPEXSITolerance.
+ * - In the case that convergence is not reached within maxPEXSIIter steps,
+ * and the update from Newton's iteration does not exceed
+ * muPEXSISafeGuard, the value of muPEXSI is the last mu plus the update
+ * from Newton's iteration.
+ * @param[out] numElectronPEXSI (global) Number of electrons
+ * evaluated at the last step.  
+ * **Note** In the case that convergence is not reached within maxPEXSIIter steps,
+ * and numElectron does not correspond to the number of electrons
+ * evaluated at muPEXSI.
+ * @param[out] muMinInertia (global) Lower bound for mu after the last
+ * inertia count procedure.
+ * @param[out] muMaxInertia (global) Upper bound for mu after the last
+ * inertia count procedure.
+ * @param[out] numTotalInertiaIter (global) Number of total inertia
+ * counting procedure.
+ * @param[out] numTotalPEXSIIter (global) Number of total %PEXSI
+ * evaluation procedure.
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ */
+void PPEXSICalculateFermiOperatorReal3(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    double            numElectronExact,
+    double*           mu,
+    double*           numElectronPEXSI,
+    int*              info );
+
+
+
+
 
 /**
  * @brief Compute the density matrices and number of electrons for a
@@ -951,17 +1053,95 @@ void PPEXSIDFTDriver(
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
     double            numElectronExact,
-		double*           muPEXSI,
-		double*           numElectronPEXSI,
+    double*           muPEXSI,
+    double*           numElectronPEXSI,
     double*           muMinInertia,
-		double*           muMaxInertia,
-		int*              numTotalInertiaIter,
-		int*              numTotalPEXSIIter,
+    double*           muMaxInertia,
+    int*              numTotalInertiaIter,
+    int*              numTotalPEXSIIter,
     int*              info );
 
 /**
  * @brief Simplified driver for solving Kohn-Sham DFT. Version 2. This
- * is still in test phase.
+ * is deprecated.
+ * 
+ * This function contains both the inertia counting step for estimating
+ * the chemical potential, and strategy for updating the chemical
+ * potential WITHOUT Newton's iteration.  Heuristics are built into this
+ * routine.  Expert users and developers can modify this routine to
+ * obtain better heuristics.  The implementation of this function
+ * contains **all** the heuristics for a DFT solver.
+ *
+ * The input parameter options are controlled through the structure
+ * PPEXSIOptions.  The default value can be obtained through
+ * PPEXSISetDefaultOptions.
+ *
+ * **Basic strategy of the heuristics**
+ *
+ * - If isInertiaCount == 1, then the inertia counting procedure is
+ * invoked until the chemical potential interval is refined from size
+ * (muMax0-muMin0) to muInertiaTolerance, or the estimated band gap is
+ * larger than muInertiaTolerance.
+ * - The number of shifts in the inertia count is always automatically
+ * chosen to be (mpisize / npPerPole). This minimizes the wall clock
+ * time of the inertia counting procedure.
+ *
+ * **Input/Output**
+ *
+ * The input H and S matrices should be given by loading functions
+ * (currently it is PPEXSILoadRealSymmetricHSMatrix).  The output
+ * matrices should be obtained from retrieving functions (currently it
+ * is PPEXSIRetrieveRealSymmetricDFTMatrix).
+ *
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[in] options (global) Other input parameters for the DFT driver.  
+ * @param[in] numElectronExact (global) Exact number of electrons, i.e.
+ * \f$N_e(\mu_{\mathrm{exact}})\f$.
+ * @param[out]  DMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero value
+ * of density matrix in CSC format.
+ * @param[out] EDMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero
+ * value of energy density matrix in CSC format.
+ * @param[out] FDMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero
+ * value of free energy density matrix in CSC format.
+ * @param[out] muPEXSI      (global) Chemical potential after the last
+ * iteration.
+ * - In the case that convergence is reached within maxPEXSIIter steps, the
+ * value of muPEXSI is the last mu used to achieve accuracy within
+ * numElectronPEXSITolerance.
+ * - In the case that convergence is not reached within maxPEXSIIter steps,
+ * and the update from Newton's iteration does not exceed
+ * muPEXSISafeGuard, the value of muPEXSI is the last mu plus the update
+ * from Newton's iteration.
+ * @param[out] numElectronPEXSI (global) Number of electrons
+ * evaluated at the last step.  
+ * **Note** In the case that convergence is not reached within maxPEXSIIter steps,
+ * and numElectron does not correspond to the number of electrons
+ * evaluated at muPEXSI.
+ * @param[out] muMinInertia (global) Lower bound for mu after the last
+ * inertia count procedure.
+ * @param[out] muMaxInertia (global) Upper bound for mu after the last
+ * inertia count procedure.
+ * @param[out] numTotalInertiaIter (global) Number of total inertia
+ * counting procedure.
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ */
+void PPEXSIDFTDriver2_Deprecate(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    double            numElectronExact,
+    double*           muPEXSI,
+    double*           numElectronPEXSI,
+    double*           muMinInertia,
+    double*           muMaxInertia,
+    int*              numTotalInertiaIter,
+    int*              info );
+
+/**
+ * @brief Simplified driver for solving Kohn-Sham DFT. Version 2.
  * 
  * This function contains both the inertia counting step for estimating
  * the chemical potential, and strategy for updating the chemical
@@ -1029,17 +1209,19 @@ void PPEXSIDFTDriver(
  */
 void PPEXSIDFTDriver2(
     PPEXSIPlan        plan,
-    PPEXSIOptions     options,
+    PPEXSIOptions*    options,
     double            numElectronExact,
-		double*           muPEXSI,
-		double*           numElectronPEXSI,
-    double*           muMinInertia,
-		double*           muMaxInertia,
-		int*              numTotalInertiaIter,
+    //int               method,
+    //int               nPoints,
+    double*           muPEXSI,
+    double*           numElectronPEXSI,
+    int*              numTotalInertiaIter,
     int*              info );
 
+
 /**
  * @brief Retrieve the output matrices after running PPEXSIDFTDriver for real input matrices.
+ * this is only used for the PEXSI method = 1
  *
  * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
  * data structure.
@@ -1055,9 +1237,9 @@ void PPEXSIDFTDriver2(
  */
 void PPEXSIRetrieveRealDFTMatrix(
     PPEXSIPlan        plan,
-		double*      DMnzvalLocal,
-		double*     EDMnzvalLocal,
-		double*     FDMnzvalLocal,
+    double*      DMnzvalLocal,
+    double*     EDMnzvalLocal,
+    double*     FDMnzvalLocal,
     double*     totalEnergyH,
     double*     totalEnergyS,
     double*     totalFreeEnergy,
@@ -1066,6 +1248,7 @@ void PPEXSIRetrieveRealDFTMatrix(
 
 /**
  * @brief Retrieve the output matrices after running PPEXSIDFTDriver for complex input matrices.
+ * this is only used for the PEXSI method = 1
  *
  * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
  * data structure.
@@ -1081,9 +1264,9 @@ void PPEXSIRetrieveRealDFTMatrix(
  */
 void PPEXSIRetrieveComplexDFTMatrix(
     PPEXSIPlan        plan,
-		double*      DMnzvalLocal,
-		double*     EDMnzvalLocal,
-		double*     FDMnzvalLocal,
+    double*      DMnzvalLocal,
+    double*     EDMnzvalLocal,
+    double*     FDMnzvalLocal,
     double*     totalEnergyH,
     double*     totalEnergyS,
     double*     totalFreeEnergy,
@@ -1163,6 +1346,159 @@ void PPEXSIGetPoleFDM(
     double        deltaE,
     double        mu );
 
+/**
+ * @brief Pole expansion for correct the complex EDM matrix
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[in] options (global) Other input parameters for the DFT driver.  
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ *
+ */
+void PPEXSICalculateEDMCorrectionComplex( 
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    int*              info );
+
+/**
+ * @brief Pole expansion for correct the complex EDM matrix
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[in] options (global) Other input parameters for the DFT driver.  
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ *
+ */
+void PPEXSICalculateEDMCorrectionReal( 
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    int*              info );
+
+/**
+ * @brief interpolate the Density matrix(DM) and Energy Density Matrix EDM
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[in] options (global) Other input parameters for the DFT driver.  
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ *
+ */
+void PPEXSIInterpolateDMReal( 
+    PPEXSIPlan        plan,
+    PPEXSIOptions*    options,
+    double            numElectronExact,
+    double            numElectronPEXSI,
+    double *          NeVec,
+    double *          muPEXSI,
+    int*              info );
+
+/**
+ * @brief interpolate the complex Density matrix(DM) and Energy Density Matrix EDM
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[in] options (global) Other input parameters for the DFT driver.  
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ *
+ */
+void PPEXSIInterpolateDMComplex( 
+    PPEXSIPlan        plan,
+    PPEXSIOptions*    options,
+    double            numElectronExact,
+    double            numElectronPEXSI,
+    double *          NeVec,
+    double *          muPEXSI,
+    int*              info );
+
+/**
+ * @brief Retrieve the output DM matrices after running PPEXSIDFTDriver for real input matrices.
+ * this is only used for the PEXSI method = 2
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[out]  DMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero value
+ * of density matrix in CSC format.
+ * @param[out]  totalEnergyH(local)  H*DM energy
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ */
+void PPEXSIRetrieveRealDM(
+    PPEXSIPlan  plan,
+    double*     DMnzvalLocal,
+    double*     totalEnergyH,
+    int*        info );
+
+/**
+ * @brief Retrieve the output DM matrices after running PPEXSIDFTDriver for real input matrices.
+ * this is only used for the PEXSI method = 2, S inverse will be computed in this procedure when
+ * it is not identity.
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[out] EDMnzvalLocal (local)  Dimension: nnzLocal.  Nonzero
+ * value of energy density matrix in CSC format.
+ * @param[out]  totalEnergyS(local)  Tr[S*EDM] energy
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ */
+void PPEXSIRetrieveRealEDM(
+    PPEXSIPlan  plan,
+    PPEXSIOptions     options,
+    double*     EDMnzvalLocal,
+    double*     totalEnergyS,
+    int*        info );
+
+
+/**
+ * @brief Retrieve the output matrices after running PPEXSIDFTDriver for complex input matrices.
+ * this is only used for the PEXSI method = 2
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[out]  DMnzvalLocal (local)  Dimension: 2*nnzLocal.  Nonzero value
+ * of density matrix in CSC format.
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ */
+void PPEXSIRetrieveComplexDM(
+    PPEXSIPlan        plan,
+    double*      DMnzvalLocal,
+    double*     totalEnergyH,
+    int*              info );
+
+/**
+ * @brief Retrieve the output matrices after running PPEXSIDFTDriver for complex input matrices.
+ * this is only used for the PEXSI method = 2, S inverse will be computed in this procedure when
+ * it is not identity.
+ *
+ * @param[in] plan (local) The plan holding the internal data structure for the %PEXSI
+ * data structure.
+ * @param[out] EDMnzvalLocal (local)  Dimension: 2*nnzLocal.  Nonzero
+ * value of energy density matrix in CSC format.
+ * @param[out] info (local) whether the current processor returns the correct information.
+ * - = 0: successful exit.  
+ * - > 0: unsuccessful.
+ */
+void PPEXSIRetrieveComplexEDM(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    double*     EDMnzvalLocal,
+    double*     totalEnergyS,
+    int*              info );
+
+
+
 #ifdef __cplusplus
 }// extern "C"
 #endif
