@@ -63,11 +63,15 @@ such enhancements or derivative works thereof, in binary and source code form.
 
 #include "pexsi/TreeBcast.hpp"
 
+#include	"pexsi/TreeBcast_v2.hpp"
+#include	"pexsi/TreeReduce_v2.hpp"
 
 #include <set>
 
 
 //#define IDX_TO_TAG(lidx,tag) (SELINV_TAG_COUNT*(lidx)+(tag)) 
+#define sym_IDX_TO_TAG( lidx, tag, numSuper, max)  ((SELINV_TAG_COUNT)*(numSuper)*((lidx)%((max)+1))+(tag))
+
 #define IDX_TO_TAG(lidx,tag,max) ((SELINV_TAG_COUNT*(lidx%(max+1))+(tag)))
 #define IDX_TO_TAG2(sidx,lidx,tag) (SELINV_TAG_COUNT*(sidx)+(tag)) 
 #define TAG_TO_IDX(tag,typetag) (((tag)-(typetag))/SELINV_TAG_COUNT) 
@@ -103,8 +107,11 @@ struct PSelInvOptions{
   /// @brief The maximum pipeline depth. 
   Int              maxPipelineDepth; 
 
+  /// @brief Use symmetric storage for the selected inversion or not. 
+  Int              symmetricStorage; 
+
   // Member functions to setup the default value
-  PSelInvOptions(): maxPipelineDepth(-1) {}
+  PSelInvOptions(): maxPipelineDepth(-1), symmetricStorage(0) {}
 };
 
 
@@ -568,15 +575,16 @@ public:
   std::vector<std::vector<Int> > ColBlockIdx_;
   std::vector<std::vector<Int> > RowBlockIdx_;
 protected:
+
+  virtual void PMatrixToDistSparseMatrix_ ( const NumVec<Int> & AcolptrLocal, const NumVec<Int> & ArowindLocal, const Int Asize, const LongInt Annz, const Int AnnzLocal, DistSparseMatrix<T>& B );
+
   // *********************************************************************
   // Variables
   // *********************************************************************
   // Data variables
 
   const GridType*       grid_;
-
   const SuperNodeType*  super_;
-
   const PSelInvOptions * options_;
   const FactorizationOptions * optionsFact_;
 
@@ -601,30 +609,44 @@ protected:
   BolNumMat                       isRecvFromCrossDiagonal_;
 
 
-#ifdef NEW_BCAST
-  std::vector<TreeBcast2<T> *> fwdToBelowTree2_; 
-  std::vector<TreeBcast2<T> *> fwdToRightTree2_; 
-#endif
+  std::vector<std::shared_ptr<TreeReduce_v2<T> > > redDTree2_; 
+
+  std::vector<std::shared_ptr<TreeBcast_v2<char> > > bcastLDataTree_; 
+  std::vector<std::shared_ptr<TreeReduce_v2<T> > > redLTree2_; 
+
   std::vector<TreeBcast *> fwdToBelowTree_; 
   std::vector<TreeBcast *> fwdToRightTree_; 
   std::vector<TreeReduce<T> *> redToLeftTree_; 
   std::vector<TreeReduce<T> *> redToAboveTree_; 
 
+  //This is for the symmetric storage implementation
+  std::vector<Int> snodeEtree_;
+  std::vector<Int> snodeTreeOffset_;
+  std::vector<std::map<Int,Int> > snodeBlkidxToTree_;
+  std::vector<std::vector<Int> > snodeTreeToBlkidx_;
+  std::list<Int> syncPoints_;
 
+  double localFlops_;
 
   struct SuperNodeBufferType{
+    //This is for the symmetric storage implementation
+    std::vector<NumMat<T> > LUpdateBufBlk;
+    std::vector< std::vector<char> > SstrLcolSendBlk;
+    std::vector< Int > SizeSstrLcolSendBlk;
+
     NumMat<T>    LUpdateBuf;
-    NumMat<T>    DiagBuf;
-    std::vector<Int>  RowLocalPtr;
-    std::vector<Int>  BlockIdxLocal;
     std::vector<char> SstrLcolSend;
+    Int               SizeSstrLcolSend;
     std::vector<char> SstrUrowSend;
     std::vector<char> SstrLcolRecv;
     std::vector<char> SstrUrowRecv;
-    Int               SizeSstrLcolSend;
     Int               SizeSstrUrowSend;
     Int               SizeSstrLcolRecv;
     Int               SizeSstrUrowRecv;
+
+    NumMat<T>    DiagBuf;
+    std::vector<Int>  RowLocalPtr;
+    std::vector<Int>  BlockIdxLocal;
     Int               Index;
     Int               Rank;
     Int               isReady;
@@ -640,13 +662,9 @@ protected:
       isReady(0){}
 
     SuperNodeBufferType(Int &pIndex) :
-      SizeSstrLcolSend(0),
-      SizeSstrUrowSend(0),
-      SizeSstrLcolRecv(0),
-      SizeSstrUrowRecv(0),
-      Index(pIndex),
-      Rank(0), 
-      isReady(0) {}
+      SuperNodeBufferType(){
+        Index = pIndex;
+      }
 
   };
 
@@ -656,6 +674,7 @@ protected:
 
   /// @brief SelInv_lookup_indexes
   inline void SelInv_lookup_indexes(SuperNodeBufferType & snode, std::vector<LBlock<T> > & LcolRecv, std::vector<UBlock<T> > & UrowRecv, NumMat<T> & AinvBuf,NumMat<T> & UBuf);
+  inline void SelInv_lookup_indexes_seq(SuperNodeBufferType & snode, std::vector<LBlock<T> > & LcolRecv, std::vector<UBlock<T> > & UrowRecv, NumMat<T> & AinvBuf,NumMat<T> & UBuf);
 
   /// @brief GetWorkSet
   inline void GetWorkSet(std::vector<Int> & snodeEtree, std::vector<std::vector<Int> > & WSet);
@@ -674,6 +693,8 @@ public:
   // Public member functions 
   // *********************************************************************
 
+  double GetTotalFlops();
+
   PMatrix();
 
   PMatrix( const GridType* g, const SuperNodeType* s, const PEXSI::PSelInvOptions * o, const PEXSI::FactorizationOptions * oFact  );
@@ -687,6 +708,8 @@ public:
 
   void Setup( const GridType* g, const SuperNodeType* s, const PEXSI::PSelInvOptions * o, const PEXSI::FactorizationOptions * oFact  );
 
+  const PSelInvOptions * Options() const { return options_; }
+
   Int NumCol() const { return super_ -> superIdx.m(); }
 
   Int NumSuper() const { return super_ ->superPtr.m() - 1; }
@@ -974,9 +997,10 @@ public:
   /// @param[in]  A Input sparse matrix to provide the sparsity pattern.
   ///
   /// @param[out] B Output sparse matrix.
+  template<typename T1 = T>
   void PMatrixToDistSparseMatrix( 
-      const DistSparseMatrix<T>& A,
-      DistSparseMatrix<T>& B );
+      const DistSparseMatrix<T1>& A, DistSparseMatrix<T>& B );
+
 
 
   /// @brief NnzLocal computes the number of nonzero elements (L and U)
@@ -998,6 +1022,9 @@ public:
 
   void CopyLU( const PMatrix & C);
   inline int IdxToTag(Int lidx, Int tag) { return SELINV_TAG_COUNT*(lidx)+(tag);}
+
+
+
 };
 
 template<typename T>  class PMatrixUnsym;
