@@ -62,12 +62,41 @@ such enhancements or derivative works thereof, in binary and source code form.
 #include	"pexsi/lapack.hpp"
 #include	"pexsi/pselinv.hpp"
 
+
+#include <memory>
 #include <set>
 
 
 
 namespace PEXSI{
 
+/// @brief CDROW returns cross diagonal processor row
+inline Int CDROW(Int pnum, const GridType* g )
+{ return (pnum % g->numProcCol) % g->numProcRow; }
+
+/// @brief CDCOL returns cross diagonal processor column
+inline Int CDCOL(Int pnum, const GridType* g )
+{ return (pnum / g->numProcCol) % g->numProcCol; }
+
+/// @brief CDPNUM returns the cross diagonal processor rank.
+inline Int CDPNUM(Int pr, Int pc, const GridType* g )
+{ return PNUM(pc,pr,g); }
+
+
+/// @brief MYCDROW returns cross diagonal processor row
+inline Int MYCDROW( const GridType* g )
+{ return MYCOL(g) % g->numProcRow; }
+
+/// @brief MYCDCOL returns cross diagonal processor column
+inline Int MYCDCOL( const GridType* g )
+{ return MYROW(g) % g->numProcCol; }
+
+/// @brief MYCDPROC returns the cross diagonal processor rank.
+inline Int MYCDPROC( const GridType* g )
+{ return PNUM(MYCDROW(g),MYCDCOL(g),g); }
+
+
+
 
 struct CDBuffers;
 
@@ -96,67 +125,45 @@ template<typename T>
 /// @brief PMatrixUnsym contains the main data structure and the
 /// computational routine for the parallel selected inversion.  
 /// 
-/// **NOTE** The following is a bit obsolete.
+/// **NOTE** 
 ///
-/// Procedure for Selected Inversion 
-/// --------------------------------
-///
-/// After factorizing a SuperLUMatrix luMat (See SuperLUMatrix for
-/// information on how to perform factorization), perform the following
-/// steps for parallel selected inversion.
-/// 
-/// - Conversion from SuperLU_DIST.
-///   
-///   Symbolic information
-///
-///       SuperNodeType super; 
-///       PMatrix PMloc;
-///       luMat.SymbolicToSuperNode( super );  
-///   
-///   Numerical information, both L and U.
-///
-///       luMat.LUstructToPMatrix( PMloc ); 
-///
-/// - Preparation.
-///
-///   Construct the communication pattern for SelInv.
-///
-///       PMloc.ConstructCommunicationPattern(); 
-///       or PMloc.ConstructCommunicationPattern_P2p(); 
-///       or PMloc.ConstructCommunicationPattern_Collectives(); 
-///   
-///   Numerical preparation so that SelInv only involves Gemm.
-///
-///       PMloc.PreSelInv();  
-///
-/// - Selected inversion.
-///
-///       PMloc.SelInv();
-///       or PMloc.SelInv_P2p();
-///       or PMloc.SelInv_Collectives();
-///
-/// - Postprocessing.
-///
-///   Get the information in DistSparseMatrix format 
-///
-///       DistSparseMatrix<Scalar> Ainv;
-///       PMloc.PMatrixToDistSparseMatrix( Ainv );  
-///
-/// Note
-/// ----
-///
-/// - All major operations of PMatrix, including the selected inversion
-/// are defined directly as the member function of PMatrix.
-///
-/// - In the current version of PMatrix, square grid is assumed.  This
-/// assumption is only used when sending the information to
-/// cross-diagonal blocks, i.e. from L(isup, ksup) to U(ksup, isup).
-/// This assumption can be relaxed later.
+///  For general non-symmetric matrices, the selected elements are
+///  :math:`\{B_{i,j}\vert A_{j,i}\ne 0\}`.  This means that the matrix
+///  elements computed corresponding to the sparsity pattern of
+///  :math:`A^T` (for more detailed explanation of the mathematical
+///  reason, please see the paper :ref:`PC2018 <citeNonSymPSelInv>`
+///  in the PEXSI documentation).  However, storing the matrix elements
+///  :math:`\{A^{-1}_{i,j}\vert A_{j,i}\ne 0\}` is practically
+///  cumbersome, especially in the context of distributed computing.
+///  Hence we choose to store the selected elements for :math:`A^{-T}`,
+///  i.e. :math:`\{A^{-T}_{i,j}\vert A_{i,j}\ne 0\}`.  These are the
+///  values obtained from the non-symmetric version of PSelInv.
+
 
 template<typename T>
   class PMatrixUnsym: public PMatrix<T>{
 
   protected:
+    virtual void PMatrixToDistSparseMatrix_( const NumVec<Int> & AcolptrLocal, const NumVec<Int> & ArowindLocal, const Int Asize, const LongInt Annz, const Int AnnzLocal, DistSparseMatrix<T>& B );
+    // NOTE: By default the returned matrix should store the transpose of the inverse matrix.
+
+  std::vector<std::vector<Int> > isSendToCD_;
+  std::vector<std::vector<Int> > isRecvFromCD_; 
+
+  std::vector<std::shared_ptr<TreeBcast_v2<char> > > bcastLStructTree_; 
+
+  std::vector<std::vector< std::shared_ptr<TreeBcast_v2<char> > > > bcastUDataTrees_; 
+  std::vector<std::shared_ptr<TreeBcast_v2<char> > > bcastUDataTree_; 
+  std::vector<std::shared_ptr<TreeBcast_v2<char> > > bcastUStructTree_; 
+
+  std::vector<std::shared_ptr<TreeReduce_v2<T> > > redUTree2_; 
+
+  std::vector<TreeReduce<T> *> redLTree_; 
+  std::vector<TreeReduce<T> *> redDTree_; 
+  std::vector<TreeReduce<T> *> redUTree_; 
+
+
+
     // *********************************************************************
     // Variables
     // *********************************************************************
@@ -193,12 +200,12 @@ template<typename T>
     };
 
 
-
-
     struct SuperNodeBufferTypeUnsym:public PMatrix<T>::SuperNodeBufferType {
       NumMat<T>    UUpdateBuf;
       std::vector<Int>  ColLocalPtr;
       std::vector<Int>  BlockIdxLocalU;
+
+      //TODO this will become unecessary
       std::vector<char> SstrLrowSend;
       std::vector<char> SstrUcolSend;
       std::vector<char> SstrLrowRecv;
@@ -207,10 +214,23 @@ template<typename T>
       Int               SizeSstrUcolSend;
       Int               SizeSstrLrowRecv;
       Int               SizeSstrUcolRecv;
+
+
+      Int isReadyL;
+      Int isReadyU;
+      bool updatesLU;
+
+      SuperNodeBufferTypeUnsym():PMatrix<T>::SuperNodeBufferType(){
+       isReadyL = 0;
+       isReadyU = 0;
+       updatesLU = false;
+      }
+
     };
 
     /// @brief SelInvIntra_P2p
     inline void SelInvIntra_P2p(Int lidx);
+    inline void SelInvIntra_New(Int lidx, Int & rank);
 
     /// @brief SelInv_lookup_indexes
     inline void SelInv_lookup_indexes(SuperNodeBufferTypeUnsym & snode,
@@ -221,6 +241,12 @@ template<typename T>
         NumMat<T> & AinvBuf,
         NumMat<T> & LBuf,
         NumMat<T> & UBuf);
+    inline void SelInv_lookup_indexes_New(SuperNodeBufferTypeUnsym & snode,
+        std::vector<LBlock<T> > & LRecv,
+        std::vector<UBlock<T> > & URecv,
+        NumMat<T> & AinvBuf,
+        NumMat<T> & LBuf,
+        NumMat<T> & UBuf);
 
     /// @brief UnpackData
     inline void UnpackData( SuperNodeBufferTypeUnsym & snode,
@@ -229,9 +255,15 @@ template<typename T>
         std::vector<UBlock<T> > & UcolRecv,
         std::vector<UBlock<T> > & UrowRecv
         );
+    inline void UnpackData_New( SuperNodeBufferTypeUnsym & snode,
+        std::vector<LBlock<T> > & LRecv,
+        std::vector<UBlock<T> > & URecv
+        );
+
 
     /// @brief ComputeDiagUpdate
     inline void ComputeDiagUpdate(SuperNodeBufferTypeUnsym & snode);
+    inline void ComputeDiagUpdate_New(SuperNodeBufferTypeUnsym & snode,bool fromU);
 
     /// @brief SendRecvCD_UpdateU
     inline void SendRecvCD(
@@ -292,6 +324,7 @@ template<typename T>
     /// pattern to be used later in the selected inversion stage.
     /// The supernodal elimination tree is used to add an additional level of parallelism between supernodes.
     void ConstructCommunicationPattern_P2p( );
+    void ConstructCommunicationPattern_New( );
 
 
     /// @brief PreSelInv prepares the structure in L_ and U_ so that
@@ -323,6 +356,7 @@ template<typename T>
     /// PreSelInv assumes that
     /// PEXSI::PMatrix::ConstructCommunicationPattern has been executed.
     virtual void PreSelInv( );
+    virtual void PreSelInv_New( );
 
     /// @brief SelInv is the main function for the selected inversion.
     ///
