@@ -153,6 +153,8 @@ private:
 
   symPACK::DistSparseMatrix<Real>     symmShiftInvRealMat_;
   symPACK::DistSparseMatrix<Complex>  symmShiftInvComplexMat_;
+
+  Int outputFileIndex_;
 #endif
 
   SuperLUOptions             luOpt_;
@@ -250,6 +252,7 @@ public:
   /// - = 2   : Detailed output.
   void SymbolicFactorizeRealSymmetricMatrix(
       Int                            solver,
+      Int                            symmetricStorage,
       std::string                    ColPerm,
       Int                            numProcSymbFact,
       Int                            verbosity );
@@ -302,6 +305,7 @@ public:
   /// - = 2   : Detailed output.
   void SymbolicFactorizeComplexSymmetricMatrix(
       Int                            solver,
+      Int                            symmetricStorage,
       std::string                    ColPerm,
       Int                            numProcSymbFact,
       Int                            verbosity );
@@ -337,26 +341,28 @@ public:
 
 
   void SelInvRealSymmetricMatrix(
-    Int               solver,
+      Int               solver,
+      Int               symmetricStorage,
       double*           AnzvalLocal,                  
       Int               verbosity,
       double*           AinvnzvalLocal );
 
   void SelInvRealUnsymmetricMatrix(
-    Int               solver,
+      Int               solver,
       double*           AnzvalLocal,                  
       Int               verbosity,
       double*           AinvnzvalLocal );
 
 
   void SelInvComplexSymmetricMatrix(
-    Int               solver,
+      Int               solver,
+      Int               symmetricStorage,
       double*           AnzvalLocal,                  
       Int               verbosity,
       double*           AinvnzvalLocal );
 
   void SelInvComplexUnsymmetricMatrix(
-    Int               solver,
+      Int               solver,
       double*           AnzvalLocal,                  
       Int               verbosity,
       double*           AinvnzvalLocal );
@@ -394,7 +400,7 @@ public:
   void CalculateNegativeInertiaReal(
       const std::vector<Real>&       shiftVec, 
       std::vector<Real>&             inertiaVec,
-    Int               solver,
+      Int                            solver,
       Int                            verbosity );
 
   /// @brief Compute the negative inertia (the number of eigenvalues
@@ -428,7 +434,7 @@ public:
   void CalculateNegativeInertiaComplex(
       const std::vector<Real>&       shiftVec, 
       std::vector<Real>&             inertiaVec,
-    Int               solver,
+      Int                            solver,
       Int                            verbosity );
 
 
@@ -466,12 +472,12 @@ public:
       Real  mu,
       Real  numElectronExact, 
       Real  numElectronTolerance,
-    Int               solver,
+      Int               solver,
       Int   verbosity,
       Real& numElectron,
       Real& numElectronDrvMu );
 
-
+#if 0
   /// @brief Compute the Fermi operator for a given chemical
   /// potential for complex Hermitian matrices.
   ///
@@ -497,6 +503,72 @@ public:
   /// @param[out] numElectron The number of electron calculated at mu.
   /// @param[out] numElectronDrvMu The derivative of the number of
   /// electron calculated with respect to the chemical potential at mu.
+  void CalculateFermiOperatorComplexDeprecate(
+      Int   numPole, 
+      Real  temperature,
+      Real  gap,
+      Real  deltaE,
+      Real  mu,
+      Real  numElectronExact, 
+      Real  numElectronTolerance,
+      Int   solver,
+      Int   verbosity,
+      Real& numElectron,
+      Real& numElectronDrvMu );
+#endif
+
+  /// @brief Compute the Fermi operator for a given chemical
+  /// potential for Hermitian Hamiltonian and overlap matrices.
+  ///
+  /// This routine also computes the single particle density matrix,
+  /// the Helmholtz free energy density matrix, and the energy density
+  /// matrix (for computing the Pulay force) simultaneously.   These
+  /// matrices can be called later via member functions DensityMatrix,
+  /// FreeEnergyDensityMatrix, EnergyDensityMatrix.
+  ///
+  /// NOTE: One should pay some special attention to the treatment of
+  /// the Hermitian case. 
+  ///
+  /// 1. Since the pole locations and weights appear in conjugate pairs
+  /// (z_l,w_l) and (conj(z_l),conj(w_l)), we do not compute the
+  /// conjugate pairs explicitly. In the real symmetric case, after
+  /// computing the density matrix rhoMat, we only need to take its
+  /// imaginary component Im(rhoMat) to obtain the correct density
+  /// matrix. In the Hermitian case, this is replaced by
+  ///   rhoMat <- 1/(2i) (rhoMat - rhoMat^*)
+  ///   where rhoMat^* is the Hermitian transpose.
+  ///
+  /// 2. The Hermitian case calls the non-symmetric version of PSelInv.
+  /// This returns the transpose of rhoMat.  Since rhoMat is Hermitian, 
+  /// we can store the correct rhoMat by applying a conjugation
+  /// operation due to the following relation.
+  ///
+  /// rhoMat = rhoMat^* = conj(rhoMat^T)
+  ///
+  /// 3. Combining 1) and 2) above, the correct density matrix can be
+  /// obtained by 
+  ///
+  ///   rhoMat <- i/2 (conj(rhoMat) - rhoMat^T) 
+  ///
+  /// The same post processing strategy should be applied to other
+  /// quantities such as the energy density matrix.
+  ///
+  /// @param[in] numPole Number of poles for the pole expansion
+  ///	@param[in] temperature  Temperature
+  /// @param[in] gap Band gap
+  /// @param[in] deltaE Upperbound of the spectrum width
+  /// @param[in] mu Initial guess of chemical potential.
+  /// @param[in] numElectronExact  Exact number of electrons.
+  /// @param[in] numElectronTolerance  Tolerance for the number of
+  /// electrons. This is just used to discard some poles in the pole
+  /// expansion.
+  /// @param[in] verbosity The level of output information.
+  /// - = 0   : No output.
+  /// - = 1   : Basic output (default)
+  /// - = 2   : Detailed output.
+  /// @param[out] numElectron The number of electron calculated at mu.
+  /// @param[out] numElectronDrvMu The derivative of the number of
+  /// electron calculated with respect to the chemical potential at mu.
   void CalculateFermiOperatorComplex(
       Int   numPole, 
       Real  temperature,
@@ -505,10 +577,13 @@ public:
       Real  mu,
       Real  numElectronExact, 
       Real  numElectronTolerance,
-    Int               solver,
+      Int   solver,
       Int   verbosity,
       Real& numElectron,
-      Real& numElectronDrvMu );
+      Real& numElectronDrvMu,
+      Int   method,
+      Int   nPoints,
+      Real  spin );
 
 
 
@@ -530,7 +605,8 @@ public:
       Real       numElectronPEXSITolerance,
       Int        matrixType,
       Int        isSymbolicFactorize,
-    Int               solver,
+      Int        solver,
+      Int        symmetricStorage,
       Int        ordering,
       Int        numProcSymbFact,
       Int        verbosity,
@@ -542,7 +618,7 @@ public:
       Int&       numTotalPEXSIIter );
 
 
-
+#if 0
   /// @brief Compute the Fermi operator and derivied quantities.
   /// 
   /// This routine also updates the chemical potential mu by reusing
@@ -583,15 +659,83 @@ public:
       Real  numElectronTolerance,
       Real  muMinPEXSI,
       Real  muMaxPEXSI,
-    Int               solver,
+      Int   solver,
       Int   verbosity,
       Real& mu,
       Real& numElectron, 
       bool& isPEXSIConverged );
+#endif
 
+  /// @brief Compute the Fermi operator and derivied quantities.
+  /// 
+  /// This routine also updates the chemical potential mu by reusing
+  /// Green's functions but with updated contour.
+  ///
+  /// This routine also computes the single particle density matrix,
+  /// the Helmholtz free energy density matrix, and the energy density
+  /// matrix (for computing the Pulay force) simultaneously.   These
+  /// matrices can be called later via member functions DensityMatrix,
+  /// FreeEnergyDensityMatrix, EnergyDensityMatrix.
+  ///
+  /// @param[in] numPole Number of poles for the pole expansion
+  ///	@param[in] temperature  Temperature
+  /// @param[in] gap Band gap
+  /// @param[in] deltaE Upperbound of the spectrum width
+  /// @param[in] numElectronExact  Exact number of electrons.
+  /// @param[in] numElectronTolerance  Tolerance for the number of
+  /// electrons. This is just used to discard some poles in the pole
+  /// expansion.
+  /// @param[in] muMinPEXSI Minimum of the interval for searching mu.
+  /// @param[in] muMaxPEXSI Maximum of the interval for searching mu.
+  /// @param[in] verbosity The level of output information.
+  /// - = 0   : No output.
+  /// - = 1   : Basic output (default)
+  /// - = 2   : Detailed output.
+  /// @param[in,out] mu Initial guess of chemical potential. On return
+  /// it gives the updated chemical potential within the range of
+  /// [muMinPEXSI, muMaxPEXSI]
+  /// @param[out] numElectron The number of electron calculated at mu.
+  /// @param[out] isConverged Whether the update strategy for finding
+  /// the chemical potential has converged.
+ 
+  void CalculateFermiOperatorReal3(
+      Int   numPole, 
+      Real  temperature,
+      Real  gap,
+      Real  deltaE,
+      Real  numElectronExact, 
+      Real  numElectronTolerance,
+      Int   solver,
+      Int   verbosity,
+      Real& mu,
+      Real& numElectron,
+      Int   method,
+      Int   nPoints, 
+      Real  spin);
+
+  /// @brief Compute the Correction of the EDM matrix.
+  void CalculateEDMCorrectionReal(
+      Int   numPole,
+      Int   solver,
+      Int   verbosity,
+      Int   nPoints, 
+      Real  spin);
+
+
+  /// @brief Compute the Correction of the EDM matrix.
+  void CalculateEDMCorrectionComplex(
+      Int   numPole,
+      Int   solver,
+      Int   verbosity,
+      Int   nPoints, 
+      Real  spin);
+
+
+
+#if 0 
   /// @brief Updated main driver for DFT. This reuses the pole
   /// expansion and only performs one PEXSI iteration per SCF step.
-  void DFTDriver2(
+  void DFTDriver2_Deprecate(
       Real       numElectronExact,
       Real       temperature,
       Real       gap,
@@ -606,7 +750,8 @@ public:
       Real       numElectronPEXSITolerance,
       Int        matrixType,
       Int        isSymbolicFactorize,
-    Int               solver,
+      Int        solver,
+      Int        symmetricStorage,
       Int        ordering,
       Int        numProcSymbFact,
       Int        verbosity,
@@ -615,9 +760,63 @@ public:
       Real&      muMinInertia,              
       Real&      muMaxInertia,             
       Int&       numTotalInertiaIter );
+#endif
+
+  /// @brief Updated main driver for DFT. This reuses the pole
+  /// expansion and only performs one PEXSI iteration per SCF step.
+  void DFTDriver2(
+      Real       numElectronExact,
+      Real       temperature,
+      Real       gap,
+      Real       deltaE,
+      Int        numPole, 
+      //Int        isInertiaCount,
+      //Real       muMin0,
+      //Real       muMax0,
+      //Real       mu0,
+      Real       muInertiaTolerance,
+      //Real       muInertiaExpansion,
+      Real       numElectronPEXSITolerance,
+      Int        matrixType,
+      Int        isSymbolicFactorize,
+      Int        solver,
+      Int        symmetricStorage,
+      Int        ordering,
+      Int        numProcSymbFact,
+      Int        verbosity,
+      Real&      muPEXSI,                   
+      Real&      numElectronPEXSI,         
+      Real&      muMinInertia,              
+      Real&      muMaxInertia,             
+      Int&       numTotalInertiaIter,
+      Int        method,
+      Int        nPoints,
+      Real       spin);
+
+  /// @brief Interpolate the DM and get the total Energy
+  void InterpolateDMReal(
+    Real                numElectronExact,
+    Real                numElectronPEXSI,
+    Real                numElectronPEXSITolerance,
+    Int                 nPoints,
+    Real              * NeVec,
+    Real              & muMin,
+    Real              & muMax,
+    Real              & muPEXSI,
+    Int                 verbosity);
 
 
-
+  /// @brief Interpolate the DM and get the total Energy
+  void InterpolateDMComplex(
+    Real                numElectronExact,
+    Real                numElectronPEXSI,
+    Real                numElectronPEXSITolerance,
+    Int                 nPoints,
+    Real              * NeVec,
+    Real              & muMin,
+    Real              & muMax,
+    Real              & muPEXSI,
+    Int                 verbosity);
 
   // *********************************************************************
   // Access data
