@@ -39,7 +39,7 @@ Enhancements, then you hereby grant the following license: a non-exclusive,
 royalty-free perpetual license to install, use, modify, prepare derivative
 works, incorporate into other computer software, distribute, and sublicense
 such enhancements or derivative works thereof, in binary and source code form.
- */
+*/
 /// @file interface.cpp
 /// @brief Interface subroutines of PPEXSI that can be called by both C and FORTRAN.
 ///
@@ -59,7 +59,7 @@ such enhancements or derivative works thereof, in binary and source code form.
 
 using namespace PEXSI;
 
-extern "C"
+  extern "C"
 void ReadDistSparseMatrixFormattedHeadInterface (
     char*    filename,
     int*     size,
@@ -68,8 +68,8 @@ void ReadDistSparseMatrixFormattedHeadInterface (
     int*     numColLocal,
     MPI_Comm comm )
 {
-  Int mpirank;  MPI_Comm_rank(comm, &mpirank);
-  Int mpisize;  MPI_Comm_size(comm, &mpisize);
+  int mpirank;  MPI_Comm_rank(comm, &mpirank);
+  int mpisize;  MPI_Comm_size(comm, &mpisize);
   std::ifstream fin;
   if( mpirank == 0 ){
     fin.open(filename);
@@ -116,7 +116,7 @@ void ReadDistSparseMatrixFormattedHeadInterface (
 // -----  end of function ReadDistSparseMatrixFormattedHeadInterface
 
 
-extern "C"
+  extern "C"
 void ReadDistSparseMatrixFormattedInterface(
     char*     filename,
     int       size,
@@ -149,7 +149,7 @@ void ReadDistSparseMatrixFormattedInterface(
 // -----  end of function ReadDistSparseMatrixFormattedInterface  
 
 
-extern "C"
+  extern "C"
 void ReadDistSparseMatrixHeadInterface (
     char*    filename,
     int*     size,
@@ -158,8 +158,8 @@ void ReadDistSparseMatrixHeadInterface (
     int*     numColLocal,
     MPI_Comm comm )
 {
-  Int mpirank;  MPI_Comm_rank(comm, &mpirank);
-  Int mpisize;  MPI_Comm_size(comm, &mpisize);
+  int mpirank;  MPI_Comm_rank(comm, &mpirank);
+  int mpisize;  MPI_Comm_size(comm, &mpisize);
   std::ifstream fin;
   if( mpirank == 0 ){
     fin.open(filename);
@@ -210,7 +210,7 @@ void ReadDistSparseMatrixHeadInterface (
 }  
 // -----  end of function ReadDistSparseMatrixHeadInterface
 
-extern "C"
+  extern "C"
 void ParaReadDistSparseMatrixInterface(
     char*     filename,
     int       size,
@@ -250,10 +250,11 @@ void ParaReadDistSparseMatrixInterface(
 extern "C"
 void PPEXSISetDefaultOptions(
     PPEXSIOptions*   options ){
+  options->spin                  = 2.0;
   options->temperature           = 0.0019;   // 300K 
   options->gap                   = 0.0;      // no gap 
   options->deltaE                = 10.0; 
-  options->numPole               = 40;
+  options->numPole               = 20;
   options->isInertiaCount        = 1;
   options->maxPEXSIIter          = 3;
   options->muMin0                = -10.0; 
@@ -262,16 +263,19 @@ void PPEXSISetDefaultOptions(
   options->muInertiaTolerance    = 0.05;
   options->muInertiaExpansion    = 0.3;      
   options->muPEXSISafeGuard      = 0.05;
-  options->numElectronPEXSITolerance = 0.01;
+  options->numElectronPEXSITolerance = 1.0E-10;
   options->matrixType            = 0;
   options->isSymbolicFactorize   = 1;
   options->solver                = 0;
+  options->symmetricStorage      = 0;
   options->ordering              = 0;
   options->rowOrdering           = 0;
   options->npSymbFact            = 1;
   options->symmetric             = 1;
   options->transpose             = 0;
+  options->method                = 2;
   options->verbosity             = 1;
+  options->nPoints               = 2;
 }   // -----  end of function PPEXSISetDefaultOptions  ----- 
 
 
@@ -283,7 +287,7 @@ PPEXSIPlan PPEXSIPlanInitialize(
     int           outputFileIndex, 
     int*          info ){
 
-  Int mpirank, mpisize;
+  int mpirank, mpisize;
   MPI_Comm_rank( comm, &mpirank );
   MPI_Comm_size( comm, &mpisize );
 
@@ -429,6 +433,9 @@ void PPEXSISymbolicFactorizeRealSymmetricMatrix(
             case 2:
               colPerm = "MMD_AT_PLUS_A";
               break;
+            case 3:
+              colPerm = "NATURAL";
+              break;
             default:
               ErrorHandling("Unsupported ordering strategy.");
               break;
@@ -440,18 +447,27 @@ void PPEXSISymbolicFactorizeRealSymmetricMatrix(
         {
           //Handle symPACK ordering options
           switch (options.ordering){
-            case 0:
-              colPerm = "PARMETIS";
+            case 5:
+              colPerm = "PTSCOTCH";
               break;
-            case 1:
-              colPerm = "METIS";
+            case 6:
+              colPerm = "SCOTCH";
               break;
             case 2:
               colPerm = "MMD";
               break;
             case 3:
+              colPerm = "NATURAL";
+              break;
+            case 4:
               colPerm = "AMD";
               break;
+            case 0:
+              colPerm = "PARMETIS";
+              break;
+            case 1:
+              colPerm = "METIS";
+              break;
             default:
               ErrorHandling("Unsupported ordering strategy.");
               break;
@@ -467,6 +483,7 @@ void PPEXSISymbolicFactorizeRealSymmetricMatrix(
     reinterpret_cast<PPEXSIData*>(plan)->
       SymbolicFactorizeRealSymmetricMatrix(
           options.solver,
+          options.symmetricStorage,
           colPerm,
           options.npSymbFact,
           options.verbosity );
@@ -505,6 +522,9 @@ void PPEXSISymbolicFactorizeRealUnsymmetricMatrix(
       case 2:
         colPerm = "MMD_AT_PLUS_A";
         break;
+      case 3:
+        colPerm = "NATURAL";
+        break;
       default:
         ErrorHandling("Unsupported ordering strategy.");
     }
@@ -573,6 +593,9 @@ void PPEXSISymbolicFactorizeComplexSymmetricMatrix(
             case 2:
               colPerm = "MMD_AT_PLUS_A";
               break;
+            case 3:
+              colPerm = "NATURAL";
+              break;
             default:
               ErrorHandling("Unsupported ordering strategy.");
               break;
@@ -584,18 +607,27 @@ void PPEXSISymbolicFactorizeComplexSymmetricMatrix(
         {
           //Handle symPACK ordering options
           switch (options.ordering){
-            case 0:
-              colPerm = "PARMETIS";
+            case 5:
+              colPerm = "PTSCOTCH";
               break;
-            case 1:
-              colPerm = "METIS";
+            case 6:
+              colPerm = "SCOTCH";
               break;
             case 2:
               colPerm = "MMD";
               break;
             case 3:
+              colPerm = "NATURAL";
+              break;
+            case 4:
               colPerm = "AMD";
               break;
+            case 0:
+              colPerm = "PARMETIS";
+              break;
+            case 1:
+              colPerm = "METIS";
+              break;
             default:
               ErrorHandling("Unsupported ordering strategy.");
               break;
@@ -612,6 +644,7 @@ void PPEXSISymbolicFactorizeComplexSymmetricMatrix(
     reinterpret_cast<PPEXSIData*>(plan)->
       SymbolicFactorizeComplexSymmetricMatrix(
           options.solver,
+          options.symmetricStorage,
           colPerm,
           options.npSymbFact,
           options.verbosity );
@@ -650,6 +683,9 @@ void PPEXSISymbolicFactorizeComplexUnsymmetricMatrix(
       case 2:
         colPerm = "MMD_AT_PLUS_A";
         break;
+      case 3:
+        colPerm = "NATURAL";
+        break;
       default:
         ErrorHandling("Unsupported ordering strategy.");
     }
@@ -783,7 +819,7 @@ void PPEXSIInertiaCountComplexMatrix(
 
 
 
-extern "C"
+  extern "C"
 void PPEXSICalculateFermiOperatorReal(
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -806,7 +842,7 @@ void PPEXSICalculateFermiOperatorReal(
         mu,
         numElectronExact,
         options.numElectronPEXSITolerance,
-          options.solver,
+        options.solver,
         options.verbosity,
         *numElectronPEXSI,
         *numElectronDrvMuPEXSI );
@@ -822,7 +858,49 @@ void PPEXSICalculateFermiOperatorReal(
   return ;
 }		// -----  end of function PPEXSICalculateFermiOperatorReal  ----- 
 
-extern "C"
+
+  extern "C"
+void PPEXSICalculateFermiOperatorReal3(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    double            numElectronExact,
+    double*           mu,
+    double*           numElectronPEXSI,
+    int*              info )
+{
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+
+  try{
+    reinterpret_cast<PPEXSIData*>(plan)->CalculateFermiOperatorReal3(
+        options.numPole,
+        options.temperature,
+        options.gap,
+        options.deltaE,
+        numElectronExact,
+        options.numElectronPEXSITolerance,
+        options.solver,
+        options.verbosity,
+        *mu,
+        *numElectronPEXSI,
+        options.method,
+        options.nPoints,
+        options.spin);
+  }
+  catch( std::exception& e )
+  {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+
+  return ;
+}		// -----  end of function PPEXSICalculateFermiOperatorReal3  ----- 
+
+
+  extern "C"
 void PPEXSICalculateFermiOperatorComplex(
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -845,10 +923,13 @@ void PPEXSICalculateFermiOperatorComplex(
         mu,
         numElectronExact,
         options.numElectronPEXSITolerance,
-          options.solver,
+        options.solver,
         options.verbosity,
         *numElectronPEXSI,
-        *numElectronDrvMuPEXSI );
+        *numElectronDrvMuPEXSI,
+        options.method,
+        options.nPoints,
+        options.spin);
   }
   catch( std::exception& e )
   {
@@ -861,8 +942,144 @@ void PPEXSICalculateFermiOperatorComplex(
   return ;
 }		// -----  end of function PPEXSICalculateFermiOperatorComplex   ----- 
 
+  extern "C"
+void PPEXSICalculateEDMCorrectionReal(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    int*              info )
+{
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
 
-extern "C"
+  try{
+    reinterpret_cast<PPEXSIData*>(plan)->CalculateEDMCorrectionReal(
+        options.numPole,
+        options.solver,
+        options.verbosity,
+        options.nPoints,
+        options.spin);
+  }
+  catch( std::exception& e )
+  {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+
+  return ;
+}		// -----  end of function PPEXSICalculateEDMCorrectionReal  ----- 
+
+  extern "C"
+void PPEXSICalculateEDMCorrectionComplex(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    int*              info )
+{
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+
+  try{
+    reinterpret_cast<PPEXSIData*>(plan)->CalculateEDMCorrectionComplex(
+        options.numPole,
+        options.solver,
+        options.verbosity,
+        options.nPoints,
+        options.spin);
+  }
+  catch( std::exception& e )
+  {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+
+  return ;
+}		// -----  end of function PPEXSICalculateEDMCorrectionComplex  ----- 
+
+  extern "C"
+void PPEXSIInterpolateDMReal(
+    PPEXSIPlan        plan,
+    PPEXSIOptions*    options,
+    double            numElectronExact,
+    double            numElectronPEXSI,
+    double *          NeVec,
+    double *          muPEXSI,
+    int*              info )
+{
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+
+  try{
+    reinterpret_cast<PPEXSIData*>(plan)->InterpolateDMReal(
+        numElectronExact,
+        numElectronPEXSI,
+        options->numElectronPEXSITolerance,
+        options->nPoints,
+        NeVec,
+        options->muMin0,
+        options->muMax0,
+        *muPEXSI, 
+        options->verbosity);
+
+  }
+  catch( std::exception& e )
+  {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+
+  return ;
+}		// -----  end of function PEXSIInterpolateDMReal  ----- 
+
+
+  extern "C"
+void PPEXSIInterpolateDMComplex(
+    PPEXSIPlan        plan,
+    PPEXSIOptions*    options,
+    double            numElectronExact,
+    double            numElectronPEXSI,
+    double *          NeVec,
+    double *          muPEXSI,
+    int*              info )
+{
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+
+  try{
+    reinterpret_cast<PPEXSIData*>(plan)->InterpolateDMComplex(
+        numElectronExact,
+        numElectronPEXSI,
+        options->numElectronPEXSITolerance,
+        options->nPoints,
+        NeVec,
+        options->muMin0,
+        options->muMax0,
+        *muPEXSI, 
+        options->verbosity);
+
+  }
+  catch( std::exception& e )
+  {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+
+  return ;
+}		// -----  end of function PEXSIInterpolateDMComplex  ----- 
+
+
+
+  extern "C"
 void PPEXSISelInvRealSymmetricMatrix (
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -876,7 +1093,8 @@ void PPEXSISelInvRealSymmetricMatrix (
 
   try{
     reinterpret_cast<PPEXSIData*>(plan)->SelInvRealSymmetricMatrix(
-          options.solver,
+        options.solver,
+        options.symmetricStorage,
         AnzvalLocal,
         options.verbosity,
         AinvnzvalLocal );
@@ -892,7 +1110,7 @@ void PPEXSISelInvRealSymmetricMatrix (
   return ;
 }		// -----  end of function PPEXSISelInvRealSymmetricMatrix  ----- 
 
-extern "C"
+  extern "C"
 void PPEXSISelInvRealUnsymmetricMatrix (
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -906,7 +1124,7 @@ void PPEXSISelInvRealUnsymmetricMatrix (
 
   try{
     reinterpret_cast<PPEXSIData*>(plan)->SelInvRealUnsymmetricMatrix(
-          options.solver,
+        options.solver,
         AnzvalLocal,
         options.verbosity,
         AinvnzvalLocal );
@@ -922,7 +1140,7 @@ void PPEXSISelInvRealUnsymmetricMatrix (
   return ;
 }		// -----  end of function PPEXSISelInvRealUnsymmetricMatrix  ----- 
 
-extern "C"
+  extern "C"
 void PPEXSISelInvComplexSymmetricMatrix (
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -936,7 +1154,8 @@ void PPEXSISelInvComplexSymmetricMatrix (
 
   try{
     reinterpret_cast<PPEXSIData*>(plan)->SelInvComplexSymmetricMatrix(
-          options.solver,
+        options.solver,
+        options.symmetricStorage,
         AnzvalLocal,
         options.verbosity,
         AinvnzvalLocal );
@@ -952,7 +1171,7 @@ void PPEXSISelInvComplexSymmetricMatrix (
   return ;
 }		// -----  end of function PPEXSISelInvComplexSymmetricMatrix  ----- 
 
-extern "C"
+  extern "C"
 void PPEXSISelInvComplexUnsymmetricMatrix (
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -966,7 +1185,7 @@ void PPEXSISelInvComplexUnsymmetricMatrix (
 
   try{
     reinterpret_cast<PPEXSIData*>(plan)->SelInvComplexUnsymmetricMatrix(
-          options.solver,
+        options.solver,
         AnzvalLocal,
         options.verbosity,
         AinvnzvalLocal );
@@ -1020,6 +1239,7 @@ void PPEXSIDFTDriver(
         options.matrixType,
         options.isSymbolicFactorize,
         options.solver,
+        options.symmetricStorage,
         options.ordering,
         options.npSymbFact,
         options.verbosity,
@@ -1040,8 +1260,9 @@ void PPEXSIDFTDriver(
   return;
 }   // -----  end of function PPEXSIDFTDriver  ----- 
 
+#if 0
 extern "C"
-void PPEXSIDFTDriver2(
+void PPEXSIDFTDriver2_Deprecate(
     /* Input parameters */
     PPEXSIPlan        plan,
     PPEXSIOptions     options,
@@ -1058,7 +1279,7 @@ void PPEXSIDFTDriver2(
     reinterpret_cast<PPEXSIData*>(plan)->GridPole();
 
   try{
-    reinterpret_cast<PPEXSIData*>(plan)->DFTDriver2(
+    reinterpret_cast<PPEXSIData*>(plan)->DFTDriver2_Deprecate(
         numElectronExact,
         options.temperature,
         options.gap,
@@ -1074,6 +1295,7 @@ void PPEXSIDFTDriver2(
         options.matrixType,
         options.isSymbolicFactorize,
         options.solver,
+        options.symmetricStorage,
         options.ordering,
         options.npSymbFact,
         options.verbosity,
@@ -1091,9 +1313,104 @@ void PPEXSIDFTDriver2(
     *info = 1;
   }
   return;
+}   // -----  end of function PPEXSIDFTDriver2_Deprecate  ----- 
+#endif
+
+extern "C"
+void PPEXSIDFTDriver2(
+    /* Input parameters */
+    PPEXSIPlan        plan,
+    PPEXSIOptions*    options,        // options is input and output
+    double            numElectronExact,
+    //int               method,
+    //int               nPoints,
+    /* Output parameters */
+    double*           muPEXSI,                   
+    double*           numElectronPEXSI,         
+    int*              numTotalInertiaIter,   
+    int*              info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+
+  try{
+    reinterpret_cast<PPEXSIData*>(plan)->DFTDriver2(
+        numElectronExact,
+        options->temperature,
+        options->gap,
+        options->deltaE,
+        options->numPole,
+        options->muInertiaTolerance,
+        options->numElectronPEXSITolerance,
+        options->matrixType,
+        options->isSymbolicFactorize,
+        options->solver,
+        options->symmetricStorage,
+        options->ordering,
+        options->npSymbFact,
+        options->verbosity,
+        *muPEXSI,
+        *numElectronPEXSI,
+        options->muMin0,
+        options->muMax0,
+        *numTotalInertiaIter,
+        options->method,
+        options->nPoints,
+        options->spin);
+  }
+  catch( std::exception& e )
+  {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
 }   // -----  end of function PPEXSIDFTDriver2  ----- 
 
+/*
+   extern "C"
+   void PPEXSIRetrieveRealDFTMatrix2(
+   PPEXSIPlan        plan,
+   double*      DMnzvalLocal,
+   double*     EDMnzvalLocal,
+   double*     FDMnzvalLocal,
+   double*     totalEnergyH,
+   double*     totalEnergyS,
+   double*     totalFreeEnergy,
+   int*              info ){
+ *info = 0;
+ const GridType* gridPole = 
+ reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+ PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
 
+ try{
+ Int nnzLocal = ptrData->RhoRealMat().nnzLocal;
+
+ blas::Copy( nnzLocal, ptrData->RhoRealMat().nzvalLocal.Data(), 1,
+ DMnzvalLocal, 1 );
+#if 0
+blas::Copy( nnzLocal, ptrData->EnergyDensityRealMat().nzvalLocal.Data(), 1,
+EDMnzvalLocal, 1 );
+
+blas::Copy( nnzLocal, ptrData->FreeEnergyDensityRealMat().nzvalLocal.Data(), 1,
+FDMnzvalLocal, 1 );
+#endif
+ *totalEnergyH = ptrData->TotalEnergyH();
+
+ *totalEnergyS = ptrData->TotalEnergyS();
+
+ *totalFreeEnergy = ptrData->TotalFreeEnergy();
+ }
+ catch( std::exception& e ) {
+ statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+ << " caught exception with message: "
+ << std::endl << e.what() << std::endl;
+ *info = 1;
+ }
+ return;
+ }   // -----  end of function PPEXSIRetrieveRealDFTMatrix2  ----- 
+ */
 
 extern "C"
 void PPEXSIRetrieveRealDFTMatrix(
@@ -1303,7 +1620,7 @@ void PPEXSIGetPoleFDM(
 // *********************************************************************
 
 /// @brief Internal subroutine to convert FORTRAN communicator to C
-extern "C" 
+  extern "C" 
 MPI_Comm f2c_comm(MPI_Fint* Fcomm)
 {
   return MPI_Comm_f2c((*Fcomm));
@@ -1311,7 +1628,7 @@ MPI_Comm f2c_comm(MPI_Fint* Fcomm)
 
 
 /// @brief FORTRAN interface for @ref ReadDistSparseMatrixFormattedHeadInterface.
-extern "C"
+  extern "C"
 void f_read_distsparsematrix_formatted_head (
     char*    filename,
     int*     size,
@@ -1333,7 +1650,7 @@ void f_read_distsparsematrix_formatted_head (
 
 
 /// @brief FORTRAN interface for @ref ReadDistSparseMatrixFormattedInterface.
-extern "C"
+  extern "C"
 void f_read_distsparsematrix_formatted (
     char*    filename,
     int      size,
@@ -1359,7 +1676,7 @@ void f_read_distsparsematrix_formatted (
 } // -----  end of function f_read_distsparsematrix_formatted  ----- 
 
 /// @brief FORTRAN interface for @ref ReadDistSparseMatrixHeadInterface.
-extern "C"
+  extern "C"
 void f_read_distsparsematrix_head (
     char*    filename,
     int*     size,
@@ -1381,7 +1698,7 @@ void f_read_distsparsematrix_head (
 
 
 /// @brief FORTRAN interface for @ref ParaReadDistSparseMatrixInterface.
-extern "C"
+  extern "C"
 void f_para_read_distsparsematrix (
     char*    filename,
     int      size,
@@ -1423,4 +1740,207 @@ PPEXSIPlan f_ppexsi_plan_initialize (
       info );
 }		// -----  end of function f_ppexsi_plan_initialize  ----- 
 
+extern "C"
+void PPEXSIRetrieveComplexDMEDM(
+    PPEXSIPlan  plan,
+    double*     DMnzvalLocal,
+    double*     EDMnzvalLocal,
+    int*        nnz,
+    int*        info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+  PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
+
+  try{
+    Int nnzLocal = ptrData->RhoComplexMat().nnzLocal;
+
+    blas::Copy( 2*nnzLocal, 
+        reinterpret_cast<double*>(ptrData->RhoComplexMat().nzvalLocal.Data()), 1,
+        DMnzvalLocal, 1 );
+
+    blas::Copy( 2*nnzLocal, 
+        reinterpret_cast<double*>(ptrData->EnergyDensityComplexMat().nzvalLocal.Data()), 1,
+        EDMnzvalLocal, 1 );
+
+    *nnz = nnzLocal;
+  }
+  catch( std::exception& e ) {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
+}   // -----  end of function PPEXSIRetrieveComplexDMEDM   ----- 
+
+
+extern "C"
+void PPEXSIRetrieveRealDMEDM(
+    PPEXSIPlan  plan,
+    double*     DMnzvalLocal,
+    double*     EDMnzvalLocal,
+    int*        nnz,
+    int*        info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+  PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
+
+  try{
+
+    Int nnzLocal = ptrData->RhoRealMat().nnzLocal;
+
+    blas::Copy( nnzLocal, ptrData->RhoRealMat().nzvalLocal.Data(), 1,
+        DMnzvalLocal, 1 );
+
+    blas::Copy( nnzLocal, ptrData->EnergyDensityRealMat().nzvalLocal.Data(), 1,
+        EDMnzvalLocal, 1 );
+
+    *nnz = nnzLocal;
+  }
+  catch( std::exception& e ) {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
+}   // -----  end of function PPEXSIRetrieveRealDMEDM   ----- 
+
+extern "C"
+void PPEXSIRetrieveRealDM(
+    PPEXSIPlan  plan,
+    double*     DMnzvalLocal,
+    double*     totalEnergyH,
+    int*              info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+  PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
+
+  try{
+    Int nnzLocal = ptrData->RhoRealMat().nnzLocal;
+
+    blas::Copy( nnzLocal, ptrData->RhoRealMat().nzvalLocal.Data(), 1,
+        DMnzvalLocal, 1 );
+    *totalEnergyH = ptrData->TotalEnergyH();
+  }
+  catch( std::exception& e ) {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
+}   // -----  end of function PPEXSIRetrieveRealDM  ----- 
+
+extern "C"
+void PPEXSIRetrieveRealEDM(
+    PPEXSIPlan  plan,
+    PPEXSIOptions     options,
+    double*     EDMnzvalLocal,
+    double*     totalEnergyS,
+    int*        info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+  PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
+
+  try{
+
+    statusOFS << std::endl << " Warning, EDM correction before retrieve" 
+      << gridPole->mpirank << std::endl;
+
+    reinterpret_cast<PPEXSIData*>(plan)->CalculateEDMCorrectionReal(
+        options.numPole,
+        options.solver,
+        options.verbosity,
+        options.nPoints,
+        options.spin);
+
+    Int nnzLocal = ptrData->EnergyDensityRealMat().nnzLocal;
+
+    blas::Copy( nnzLocal, ptrData->EnergyDensityRealMat().nzvalLocal.Data(), 1,
+        EDMnzvalLocal, 1 );
+
+    *totalEnergyS = ptrData->TotalEnergyS();
+  }
+  catch( std::exception& e ) {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
+}   // -----  end of function PPEXSIRetrieveRealEDM   ----- 
+
+extern "C"
+void PPEXSIRetrieveComplexDM(
+    PPEXSIPlan        plan,
+    double*      DMnzvalLocal,
+    double*     totalEnergyH,
+    int*              info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+  PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
+
+  try{
+    Int nnzLocal = ptrData->RhoComplexMat().nnzLocal;
+
+    blas::Copy( 2*nnzLocal, 
+        reinterpret_cast<double*>(ptrData->RhoComplexMat().nzvalLocal.Data()), 1,
+        DMnzvalLocal, 1 );
+
+    *totalEnergyH = ptrData->TotalEnergyH();
+  }
+  catch( std::exception& e ) {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
+}   // -----  end of function PPEXSIRetrieveComplexDM  ----- 
+
+extern "C"
+void PPEXSIRetrieveComplexEDM(
+    PPEXSIPlan        plan,
+    PPEXSIOptions     options,
+    double*     EDMnzvalLocal,
+    double*     totalEnergyS,
+    int*              info ){
+  *info = 0;
+  const GridType* gridPole = 
+    reinterpret_cast<PPEXSIData*>(plan)->GridPole();
+  PPEXSIData* ptrData = reinterpret_cast<PPEXSIData*>(plan);
+
+  try{
+    statusOFS << std::endl << " Warning, EDM correction before retrieve" 
+      << gridPole->mpirank << std::endl;
+
+    reinterpret_cast<PPEXSIData*>(plan)->CalculateEDMCorrectionComplex(
+        options.numPole,
+        options.solver,
+        options.verbosity,
+        options.nPoints,
+        options.spin);
+
+    Int nnzLocal = ptrData->RhoComplexMat().nnzLocal;
+
+    blas::Copy( 2*nnzLocal, 
+        reinterpret_cast<double*>(ptrData->EnergyDensityComplexMat().nzvalLocal.Data()), 1,
+        EDMnzvalLocal, 1 );
+
+    *totalEnergyS = ptrData->TotalEnergyS();
+  }
+  catch( std::exception& e ) {
+    statusOFS << std::endl << "ERROR!!! Proc " << gridPole->mpirank 
+      << " caught exception with message: "
+      << std::endl << e.what() << std::endl;
+    *info = 1;
+  }
+  return;
+}   // -----  end of function PPEXSIRetrieveComplexEDM ----- 
 
