@@ -53,8 +53,8 @@ such enhancements or derivative works thereof, in binary and source code form.
 
 #include <Cnames.h>
 extern "C"{ void
-#ifdef SLU_MAJOR_VERSION
-#if SLU_MAJOR_VERSION < 5
+#ifdef SUPERLU_DIST_MAJOR_VERSION
+#if SUPERLU_DIST_MAJOR_VERSION < 5
 pdsymbfact(superlu_options_t *options, SuperMatrix *A, 
     ScalePermstruct_t *ScalePermstruct, gridinfo_t *grid,
     LUstruct_t *LUstruct, SuperLUStat_t *stat, int *numProcSymbFact,
@@ -112,25 +112,7 @@ RealGridData & RealGridData::operator = (const RealGridData & g)
 }
 
 void RealGridData::GridInit( MPI_Comm comm, Int nprow, Int npcol ){
-#ifdef SWAP_ROWS_COLS
-  int * usermap = new int[nprow*npcol];
-  for(int mpirank = 0;mpirank<nprow*npcol;mpirank++){
-    int myrow = mpirank % npcol;
-    int mycol = mpirank / npcol;
-    usermap[myrow*npcol+mycol] = mpirank;
-  }
-  superlu_gridmap(
-      comm, nprow, npcol,
-      usermap, /* usermap(i,j) holds the process
-                  number to be placed in {i,j} of
-                  the process grid.  */
-      npcol, &info_->grid);
-  delete [] usermap;
-#else
   superlu_gridinit(comm, nprow, npcol, &info_->grid);
-#endif
-
-
 }
 
 void RealGridData::GridExit(  ){
@@ -158,8 +140,8 @@ protected:
   /// 
   /// to make sure that symmetric permutation is used.
   ///
-#ifdef SLU_MAJOR_VERSION
-#if SLU_MAJOR_VERSION < 5
+#ifdef SUPERLU_DIST_MAJOR_VERSION
+#if SUPERLU_DIST_MAJOR_VERSION < 5
   superlu_options_t   options;                  
 #else
   superlu_dist_options_t   options;                  
@@ -222,7 +204,15 @@ RealSuperLUData_internal::RealSuperLUData_internal(const SuperLUGrid<Real>& g, c
   // The default value of ColPerm uses the default value from SuperLUOptions
   options.Fact              = DOFACT;
   if(opt.RowPerm == "LargeDiag"){
+#ifdef SUPERLU_DIST_MAJOR_VERSION
+#  if SUPERLU_DIST_MAJOR_VERSION < 6 
     options.RowPerm         = LargeDiag;
+#  else
+    options.RowPerm         = LargeDiag_MC64;
+#  endif
+#else
+    options.RowPerm         = LargeDiag;
+#endif
   }
   else{
     options.RowPerm         = NOROWPERM;
@@ -245,6 +235,9 @@ RealSuperLUData_internal::RealSuperLUData_internal(const SuperLUGrid<Real>& g, c
   // Necessary to invoke static scheduling of SuperLU
   options.lookahead_etree   = YES;
   options.SymPattern        = YES;
+//#ifdef _PRINT_STATS_
+//  options.PrintStat         = YES;
+//#endif
 
   if(opt.Symmetric == 1){
     options.RowPerm         = NOROWPERM;
@@ -406,8 +399,8 @@ void RealSuperLUData::DistSparseMatrixToSuperMatrixNRloc( DistSparseMatrix<Real>
   }
   gridinfo_t* grid = ptrData->grid;
 
-  Int mpirank = grid->iam;
-  Int mpisize = grid->nprow * grid->npcol;
+  int mpirank = grid->iam;
+  int mpisize = grid->nprow * grid->npcol;
 
   int_t *colindLocal, *rowptrLocal;
   double *nzvalLocal;
@@ -416,8 +409,7 @@ void RealSuperLUData::DistSparseMatrixToSuperMatrixNRloc( DistSparseMatrix<Real>
   Int firstRow = mpirank * numRowLocalFirst;
   Int numRowLocal = -1;
   Int nnzLocal = -1;
-
-  if(options.Transpose == 1  || options.Symmetric == 1 ){
+  if(options.Transpose == 1 || options.Symmetric == 1 ){
     numRowLocal = sparseA.colptrLocal.m() - 1;
     nnzLocal = sparseA.nnzLocal;
 
@@ -431,7 +423,6 @@ void RealSuperLUData::DistSparseMatrixToSuperMatrixNRloc( DistSparseMatrix<Real>
         colindLocal );
     std::copy( sparseA.nzvalLocal.Data(), sparseA.nzvalLocal.Data() + sparseA.nzvalLocal.m(),
         nzvalLocal );
-
   }
   else{
     DistSparseMatrix<Real> sparseB;
@@ -453,7 +444,6 @@ void RealSuperLUData::DistSparseMatrixToSuperMatrixNRloc( DistSparseMatrix<Real>
   }
 
 
-
   // Important to adjust from FORTRAN convention (1 based) to C convention (0 based) indices
   for(Int i = 0; i < sparseA.rowindLocal.m(); i++){
     colindLocal[i]--;
@@ -583,7 +573,13 @@ RealSuperLUData::NumericalFactorize	(  )
   statusOFS<<"******************SUPERLU STATISTICS****************"<<std::endl;  
   statusOFS<<"Number of tiny pivots: "<<ptrData->stat.TinyPivots<<std::endl;  
   statusOFS<<"Number of look aheads: "<<ptrData->stat.num_look_aheads<<std::endl;  
-  statusOFS<<"Number of FLOPS during factorization: "<<ptrData->stat.ops[FACT]<<std::endl;  
+  
+  float flopcnt = 0;
+  MPI_Allreduce(&ptrData->stat.ops[FACT], &flopcnt, 1, MPI_FLOAT, MPI_SUM, ptrData->grid->comm);
+  statusOFS<<"Number of FLOPS for factorization: "<<flopcnt<<std::endl;  
+  if(!ptrData->grid->iam){
+    std::cout<<"Total FLOPs for factorization is "<<flopcnt<<std::endl;
+  }
   statusOFS<<"****************************************************"<<std::endl;  
 #endif
 
@@ -890,6 +886,8 @@ RealSuperLUData::LUstructToPMatrix	( PMatrix<Real>& PMloc )
     }  // if(index)
   } // for(jb)
 
+        if(PMloc.Options()!=nullptr){
+          if (PMloc.Options()->symmetricStorage!=1){
   // U part
 #if ( _DEBUGlevel_ >= 1 )
   statusOFS << std::endl << "LUstructToPMatrix::U part" << std::endl;
@@ -969,6 +967,8 @@ RealSuperLUData::LUstructToPMatrix	( PMatrix<Real>& PMloc )
     } // if( index )
 
   } // for(ib)
+          }
+        }
 
   for( Int ib = 0; ib < PMloc.NumLocalBlockRow(); ib++ ){
     std::vector<Int> & rowBlockIdx = PMloc.RowBlockIdx(ib);
